/*
	Program:	3DS reader
	Purpose:	Provide a modular parser for 3DS files
	Author:		Geshem ( erez27@gmail.com )

	Special thanks to:
		http://www.the-labs.com/Blender/3DS-details.html	- for providing me with lotsa info
*/


#include "coredefs.h"
#include <stdlib.h>
#include <stdio.h>


#define MAX_VERTICES	(65536)
#define MAX_OBJBLOCKS	(256)

#define	DUMP3DS_INDENT	(4)


typedef void chunk_handler_func_t(FILE* f, int ind, long limit);

#define SKIP_CHUNK	((chunk_handler_func_t*)(-1))

typedef struct chunk_handler {
    word id;
    chunk_handler_func_t* func;
} chunk_handler;
	
typedef struct chunk_name {
	word id;
	char* name;
} chunk_name;


#pragma pack(2)
struct chunk_header {
    word    id;
    dword   len;
};
#pragma pack()





/* Enumeration of chunk IDs to be used by chunk lists */
enum {
	CHUNK_NULL		= 0x0000,
    CHUNK_RGBF      = 0x0010,
    CHUNK_RGBB      = 0x0011,
	CHUNK_M3D_VERSION = 0x0002,
/*    CHUNK_RBGB2     = 0x0012,       // ?? NOT HLS.    */

    CHUNK_MAIN      = 0x4D4D,
        CHUNK_OBJMESH   = 0x3D3D,
			CHUNK_MESH_VERSION	= 0x3d3e,
			CHUNK_MASTER_SCALE	= 0x100,
            CHUNK_BKGCOLOR  = 0x1200,
            CHUNK_AMBCOLOR  = 0x2100,
            CHUNK_OBJBLOCK  = 0x4000,
                CHUNK_TRIMESH   = 0x4100,
                    CHUNK_VERTLIST  = 0x4110,
                    CHUNK_FACELIST  = 0x4120,
                    CHUNK_FACEMAT   = 0x4130,
                    CHUNK_MAPLIST   = 0x4140,
                    CHUNK_SMOOLIST  = 0x4150,
                    CHUNK_TRMATRIX  = 0x4160,
                CHUNK_LIGHT     = 0x4600,
                    CHUNK_SPOTLIGHT = 0x4610,
                CHUNK_CAMERA    = 0x4700,
        CHUNK_MATERIAL  = 0xAFFF,
            CHUNK_MATNAME   = 0xA000,
            CHUNK_AMBIENT   = 0xA010,
            CHUNK_DIFFUSE   = 0xA020,
            CHUNK_SPECULAR  = 0xA030,
            CHUNK_TEXTURE   = 0xA200,
            CHUNK_BUMPMAP   = 0xA230,
            CHUNK_MAPFILE   = 0xA300,
        CHUNK_KEYFRAMER = 0xB000,
			CHUNK_KFHDR		= 0xb00a,
            CHUNK_FRAMES    = 0xB008,
				CHUNK_KFCURTIME	= 0xb009,
				CHUNK_OBJECT_NODE_TAG	= 0xb002,
				CHUNK_CAMERA_NODE_TAG	= 0xb003,
				CHUNK_TARGET_NODE_TAG	= 0xb004,
					CHUNK_BOUNDBOX		= 0xb014,
					CHUNK_NODE_ID		= 0xB030,
      				CHUNK_NODE_HDR 		= 0xB010,
					CHUNK_PIVOT			= 0xB013,
					CHUNK_INSTANCE_NAME	= 0xb011,
					CHUNK_POS_TRACK_TAG	= 0xB020,
					CHUNK_ROT_TRACK_TAG	= 0xB021,
					CHUNK_SCL_TRACK_TAG	= 0xB022,
					CHUNK_FOV_TRACK_TAG = 0xB023,
					CHUNK_ROL_TRACK_TAG = 0xB024

};


/* Main function */
void process_3ds(char* filename, chunk_handler* handlers_list);

/* Handlers list provided with the module which only prints data to stdout. Useful for debugging. */
extern chunk_handler print3ds_handlers_list[];


#define m3d_fread(i,f) fread(&i,sizeof(i), 1, f)
#define M3D_REPORT_CHUNK_DATA(args) { if(printf_3ds) { printf("%*c",ind, ' '); printf args; (void)putchar('\n'); } }
#define m3d_fread_ext(i,f) { if (m3d_fread(i,f) != 1) {m3d_report_error(f); return;} }

#define NAME_BUFFER_SIZE 256
extern char name_buffer[NAME_BUFFER_SIZE];

/* Read a C string (ASCIIZ) from file 'f' into global var 'name_buffer'.
 * Stops reading on null-terminator or end-of-file.
 * Stops storing if length exceeds maximum (defined by NAME_BUFFER_SIZE) */
extern void m3d_fread_name(FILE* f);

