#include <vector>
#include <math.h>
#include <SDL/SDL_opengl.h>
#include <GL/glut.h>
#include <iostream>

#include "Ptr.h"
#include "CustomSceneNodes.h"

namespace GameEngine {

	using namespace std;

	void DrawableSceneNode::render() {
		glPushMatrix();
		applyTransform();
		drawableObject_->draw();
		renderChildren();
		glPopMatrix();
	}
/*
	ModelSceneNode::ModelSceneNode(Ptr<Model> model) : DrawableSceneNode(model) {
		//nameIs(model->name() + "_SN");
	}
*/
	void ModelSceneNode::applyTransform() {
		SceneNode::applyTransform();
		Ptr<Model> model = ptr_cast<Model>(drawableObject_);
		model->normTransform()->apply();
	}
/*
	void ModelSceneNode::render() {
		//this->rotate(5, 1, 2, 3);
		//this->rotate(1, 1, 1, 1);
		glPushMatrix();
		applyTransform();
		model_->draw();
		renderChildren();
		glPopMatrix();
	}
*/
	TerrainSceneNode::TerrainSceneNode(Ptr<Terrain> terrain) : SceneNode("Terrain_SN") {
		terrain_ = terrain;
	}

	void TerrainSceneNode::render() {
		glPushMatrix();

		applyTransform();
/*
		// draw axis
		glColor3f(0.7, 0.7, 0.7);
		glBegin(GL_LINES);
		glVertex2f(-10, 0);
		glVertex2f(10, 0);

		glVertex2f(0, -10);
		glVertex2f(0, 10);
		glEnd();
*/
		//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		terrain_->draw();
		//glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		renderChildren();

		glPopMatrix();
	}

	void CubeSceneNode::render() {
		this->rotate(5, 1, 1, 0);
		glPushMatrix();
		applyTransform();

		Ptr<Vertex3> v1, v2, v3;
		Ptr<Vec3> n;
		v1 = new Vertex3; v2 = new Vertex3; v3 = new Vertex3; n = new Vec3;
		glBegin(GL_QUADS);		// Draw The Cube Using quads

		glEnable(GL_LIGHTING);
		glEnable(GL_COLOR_MATERIAL);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

		glColor3f(0.0f,1.0f,0.0f);	// Color Blue
		v1->x = 1; v1->y = 1; v1->z = -1;
		v2->x = -1; v2->y = 1; v2->z = -1;
		v3->x = -1; v3->y = 1; v3->z = 1;
		n = Math::computeNormal(v1, v2, v3);
		glNormal3f( n->x, n->y, n->z );
		//glNormal3f( 0.0f, 1.0f, 0.0f);
		glVertex3f( 0.5f, 0.5f,-0.5f);	// Top Right Of The Quad (Top)
		glVertex3f(-0.5f, 0.5f,-0.5f);	// Top Left Of The Quad (Top)
		glVertex3f(-0.5f, 0.5f, 0.5f);	// Bottom Left Of The Quad (Top)
		glVertex3f( 0.5f, 0.5f, 0.5f);	// Bottom Right Of The Quad (Top)

		glColor3f(1.0f,0.5f,0.0f);	// Color Orange
		v1->x = 1; v1->y = -1; v1->z = 1;
		v2->x = -1; v2->y = -1; v2->z = 1;
		v3->x = -1; v3->y = -1; v3->z = -1;
		n = Math::computeNormal(v1, v2, v3);
		glNormal3f( n->x, n->y, n->z );
		//glNormal3f( 0.0f,-1.0f, 0.0f);
		glVertex3f( 0.5f,-0.5f, 0.5f);	// Top Right Of The Quad (Bottom)
		glVertex3f(-0.5f,-0.5f, 0.5f);	// Top Left Of The Quad (Bottom)
		glVertex3f(-0.5f,-0.5f,-0.5f);	// Bottom Left Of The Quad (Bottom)
		glVertex3f( 0.5f,-0.5f,-0.5f);	// Bottom Right Of The Quad (Bottom)

		glColor3f(1.0f,0.0f,0.0f);	// Color Red
		v1->x = 1; v1->y = 1; v1->z = 1;
		v2->x = -1; v2->y = 1; v2->z = 1;
		v3->x = -1; v3->y = -1; v3->z = 1;
		n = Math::computeNormal(v1, v2, v3);
		glNormal3f( n->x, n->y, n->z );
		//glNormal3f( 0.0f, 0.0f, 1.0f);
		glVertex3f( 0.5f, 0.5f, 0.5f);	// Top Right Of The Quad (Front)
		glVertex3f(-0.5f, 0.5f, 0.5f);	// Top Left Of The Quad (Front)
		glVertex3f(-0.5f,-0.5f, 0.5f);	// Bottom Left Of The Quad (Front)
		glVertex3f( 0.5f,-0.5f, 0.5f);	// Bottom Right Of The Quad (Front)

		glDisable(GL_COLOR_MATERIAL);
		glDisable(GL_LIGHTING);

		glColor3f(1.0f,1.0f,0.0f);	// Color Yellow
		glNormal3f( 0.0f, 0.0f,-1.0f);
		glVertex3f( 0.5f,-0.5f,-0.5f);	// Top Right Of The Quad (Back)
		glVertex3f(-0.5f,-0.5f,-0.5f);	// Top Left Of The Quad (Back)
		glVertex3f(-0.5f, 0.5f,-0.5f);	// Bottom Left Of The Quad (Back)
		glVertex3f( 0.5f, 0.5f,-0.5f);	// Bottom Right Of The Quad (Back)

		glColor3f(0.0f,0.0f,1.0f);	// Color Blue
		glNormal3f(-1.0f, 0.0f, 0.0f);
		glVertex3f(-0.5f, 0.5f, 0.5f);	// Top Right Of The Quad (Left)
		glVertex3f(-0.5f, 0.5f,-0.5f);	// Top Left Of The Quad (Left)
		glVertex3f(-0.5f,-0.5f,-0.5f);	// Bottom Left Of The Quad (Left)
		glVertex3f(-0.5f,-0.5f, 0.5f);	// Bottom Right Of The Quad (Left)

		glColor3f(1.0f,0.0f,1.0f);	// Color Violet
		glNormal3f( 1.0f, 0.0f, 0.0f);
		glVertex3f( 0.5f, 0.5f,-0.5f);	// Top Right Of The Quad (Right)
		glVertex3f( 0.5f, 0.5f, 0.5f);	// Top Left Of The Quad (Right)
		glVertex3f( 0.5f,-0.5f, 0.5f);	// Bottom Left Of The Quad (Right)
		glVertex3f( 0.5f,-0.5f,-0.5f);	// Bottom Right Of The Quad (Right)
		glEnd();			// End Drawing The Cube}

		// call superclass (which renders children)
		renderChildren();

		glPopMatrix();
	}

	void SphereSceneNode::render() {
		glPushMatrix();
		applyTransform();

		glColor3f(1.0f, 0.0f, 0.0f);
		//float color[4] = {1.0f, 0.0f, 0.0f, 1.0f};
		//glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color);
		glutSolidSphere(0.5f, 20, 20);

		renderChildren();

		glPopMatrix();
	}

	void LightSceneNode::render() {
		this->rotate(5, 0, 1, 0);
		this->translate(1, 0, 0);
		glPushMatrix();
		applyTransform();

		//glDisable(GL_LIGHTING);
		glColor3f(1, 1, 0);
		glutSolidSphere(0.5f, 20, 20);
		//glEnable(GL_LIGHTING);

		float lightPos[4] = {0.0f, 0.0f, 0.0f, 1.0f};
		glLightfv(GL_LIGHT0, GL_POSITION, lightPos);

		renderChildren();

		glPopMatrix();
	}
}
