#ifndef ENGINE_H
#define ENGINE_H

#include <math.h>
#include <string>
#include <vector>
#include <iostream>
#include <SDL/SDL.h>

#include "Ptr.h"
#include "PtrInterface.h"
#include "Model.h"
#include "SceneNode.h"
#include "Picker.h"
#include "Terrain.h"
#include "Timer.h"
#include "HUD.h"

namespace GameEngine {

	using namespace Fwk;
	using namespace std;

	#define FOV_ANGLE       45

	#define WINDOW_WIDTH    1024
	#define WINDOW_HEIGHT   768
	#define WINDOW_X        100
	#define WINDOW_Y        100
	#define WINDOW_TITLE    "Game Engine"
	#define WINDOW_BPP		32

	#define CENTER_X        0.0
	#define CENTER_Y        0.0
	#define CENTER_Z        0.0

	#define VIEWER_X        0.0
	#define VIEWER_Y        -4.0
	#define VIEWER_Z        20.0

	#define UP_X            0.0
	#define UP_Y            1.0
	#define UP_Z            0.0

	#define CLIPPLANE_NEAR  1
	#define CLIPPLANE_FAR   1000.0

	#define FRAMES_PER_SECOND 30

	#define LIGHT_X			20.0
	#define LIGHT_Y			0.0
	#define LIGHT_Z			20.0

	class Engine : public PtrInterface<Engine> {
	public:
		virtual void run(int argc, char **argv);

	protected:
		Engine();

		Ptr<SceneNode> rootSN_;
		Ptr<Terrain> terrain_;
		vector<Ptr<Model> > models_;
		vector<Ptr<Material> > materials_;
		Ptr<Picker> picker_;
		Ptr<FPS> fps_;
		Ptr<HUD> hud_;

		virtual void initAll();
		virtual void initDisplay();
		virtual void initGLSettings();
		virtual void initLighting();
		virtual void initTextures();
		virtual void initMatrixes();
		virtual void initMyStuff();

		virtual void exitEngine();

		void handleEvents();
		virtual void handleEvent(SDL_Event &e);
		virtual void onMouseMotion(unsigned int x, unsigned int y);
		virtual void onMouseBtnDown(unsigned int button, unsigned int x, unsigned int y);
		virtual void onMouseBtnUp(unsigned int button, unsigned int x, unsigned int y);
		virtual void onKeyDown(SDLKey key, SDLMod modifer);
		virtual void onKeyUp(SDLKey key, SDLMod modifer);

		void gameLoop();
		virtual void draw();
		virtual void update(float elapsedMillis);
		void resize(int w, int h);

		unsigned int processPick(bool terrainOnly, unsigned int x, unsigned int y);
		virtual void onPick(unsigned int pickedId);

		// soft shadows
		void initSoftShadows();
		void renderWithShadows();
		void testScene();
		bool drawSoftShadows_;
		Ptr<Vec3> cameraPosition_;
		Ptr<Vec3> lightPosition_;
		int shadowMapSize_;
		GLuint shadowMapTexture_;
		Ptr<Matrix> lightProjectionMatrix_, lightViewMatrix_;
		Ptr<Matrix> cameraProjectionMatrix_, cameraViewMatrix_;
	};

}

#endif