#ifndef HUD_H
#define HUD_H

#include <vector>
#include <GL/glut.h>

#include "Ptr.h"
#include "PtrInterface.h"
#include "Drawable.h"
#include "Color.h"
#include "Texture.h"

namespace GameEngine {

	using namespace Fwk;
	using namespace std;

	class MouseListener : public PtrInterface<MouseListener> {
	public:
		virtual bool onMouseDown(int sdlButton, int x, int y) = 0;
		virtual bool onMouseUp(int sdlButton, int x, int y) = 0;
		virtual bool onMouseMotion(int x, int y) = 0;
	};

	class Container : public Drawable {
	public:
		// satisfy drawable
		virtual void build() { };
		virtual void draw();

		Ptr<Container> parent() { return parent_; }
		float x() { return x_; }
		float y() { return y_; }
		float width() { return width_; }
		float height() { return height_; }
		Ptr<Color4> backgroundColor() { return backgroundColor_; }
		bool contains(float x, float y);
		bool visible() { return visible_; }

		void childNew(Ptr<Container> child) { children_.push_back(child); }
		void xIs(float x) { x_ = x; }
		void yIs(float y) { y_ = y; }
		void widthIs(float width) { width_ = width; }
		void heightIs(float height) { height_ = height; }
		void backgroundColorIs(Ptr<Color4> color) { backgroundColor_ = color; }
		void visibleIs(bool v) { visible_ = v; }

		virtual void onDel() { }
		void clear();

	protected:
		Container(Ptr<Container> parent) :
		  parent_(parent),
		  children_(),
		  x_(0),
		  y_(0),
		  width_(0),
		  height_(0),
		  backgroundColor_(Color4::White()) {
			if (parent) {
				parent->childNew(this);
			}
		}

		Ptr<Container> parent_;
		vector<Ptr<Container> > children_;
		float x_, y_;
		float width_, height_;
		Ptr<Color4> backgroundColor_;
		bool visible_;
	};

	class Panel : public Container {
	public:
		Panel(Ptr<Container> parent) : Container(parent) { }

	protected:
	};

	class Component : public Container {
	public:
		void* glutFont() { return glutFont_; }
		string text() { return text_; }
		Ptr<Texture> texture() { return texture_; }

		void fontIs(void* glutFont) { glutFont_ = glutFont; }
		void textIs(string text) { text_ = text; }
		void textxIs(float textx) { textx_ = textx; }
		void textyIs(float texty) { texty_ = texty; }
		void textureIs(Ptr<Texture> texture) { texture_ = texture; }

		virtual void draw();

		int textHeight();

	protected:
		Component(Ptr<Container> parent) :
			Container(parent),
			text_(),
			textx_(0),
			texty_(0),
			glutFont_(GLUT_BITMAP_HELVETICA_18) {
			texture_ = NULL;
		}

		void* glutFont_;
		string text_;
		float textx_, texty_;
		Ptr<Texture> texture_;

		void drawText();
	};

	class Label : public Component {
	public:
		Label(Ptr<Container> parent) :
			Component(parent) { }

	protected:
	};

	class Button : public Component {
	public:
		Button(Ptr<Container> parent) :
		  Component(parent),
		  hover_(false) {
		}

	protected:
		bool hover_;

		virtual void onMouseEnter(int x, int y) { };
		virtual void onMouseExit(int x, int y) { };
	};

	class HUD : public Drawable {
	public:
		HUD() :
		  Drawable(),
		  winWidth_(0),
		  winHeight_(0) {
			rootPanel_ = new Panel(NULL);
		}
		HUD(int winWidth, int winHeight) :
		  Drawable(),
		  winWidth_(winWidth),
		  winHeight_(winHeight),
		  mouseListeners_() {
			rootPanel_ = new Panel(NULL);
		}

		Ptr<Panel> rootPanel() { return rootPanel_; }
		void mouseListenerNew(Ptr<MouseListener> listener);
		void mouseListenerDel(Ptr<MouseListener> listener);

		bool onMouseDown(int sdlButton, int x, int y);
		bool onMouseUp(int sdlButton, int x, int y);
		bool onMouseMotion(int x, int y);

	protected:
		Ptr<Panel> rootPanel_;
		int winWidth_;
		int winHeight_;
		vector<Ptr<MouseListener> > mouseListeners_;
		vector<Ptr<MouseListener> > deletedMouseListeners_;

		void setupOrthoView(int x, int y);
		void setupPerspectiveView();
		void deleteDeletedMouseListeners();
	};

}

#endif
