#ifndef MODEL_H
#define MODEL_H

#include <string>
#include <vector>
#include <map>

#include "Texture.h"
#include "Printable.h"
#include "Drawable.h"
#include "Geometry.h"
#include "SceneNode.h"
#include "Ptr.h"
#include "PtrInterface.h"

namespace GameEngine {

	using namespace std;
	using namespace Fwk;

	class Model : public Drawable, public Printable {
	public:
		Model() : Drawable(), meshes_(), materials_() {
			normTransform_ = new Matrix;
			sceneNode_ = NULL;
		}
		Model(string name) : Drawable(name), meshes_(), materials_() {
			normTransform_ = new Matrix;
			sceneNode_ = NULL;
		}

		// satisfy Drawable
		void draw();
		void build();
		Ptr<Drawable> copy();

		// satisfy Printable
		string toString();

		// accessors
		// TODO: index or unique id?
		Ptr<Mesh> mesh(int i) { return meshes_[i]; }
		Ptr<Material> material(string name) { return materials_[name]; }

		// mutators
		void sceneNodeIs(Ptr<SceneNode> sceneNode) { sceneNode_ = sceneNode; }
		void meshNew(Ptr<Mesh> mesh) { meshes_.push_back(mesh);	}
		void materialNew(string name, Ptr<Material> material) { materials_[name] = material; }

		// normalizing
		void normalize();
		Ptr<Matrix> normTransform() { return normTransform_; }
		Ptr<SceneNode> sceneNode() { return sceneNode_; }

		// materials and textures
		vector<Ptr<Material> > loadMaterials();

	protected:
		vector<Ptr<Mesh> > meshes_;
		map<string, Ptr<Material> > materials_;
		Ptr<Matrix> normTransform_;
		Ptr<SceneNode> sceneNode_;
	};

}

#endif