#include <stdio.h>
#include <iostream>
#include <sstream>
#include <vector>
#include <string>

#include "ModelReader3DS.h"

//using namespace GameEngine;
using namespace M3DS;
using namespace std;

/*
void ModelReader3DS::chunkM3dVersionReader(FILE* f, unsigned int indent, unsigned long p) {
	word version;
	M3D_FREAD_EXT(version, f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",version));
}
*/
void ModelReader3DS::chunkRGBFReader(FILE* f, unsigned int indent, unsigned long p) {
	float rgb[3];
	M3D_FREAD_EXT(rgb, f);	
    M3D_REPORT_CHUNK_DATA(indent, verbose,
		("Red: %f, Green: %f, Blue: %f", rgb[0], rgb[1], rgb[2]));

	// add color to material
	Ptr<Material> curr = currentMaterial();
	if (lightColorChunkId_ == CHUNK_AMBIENT) {
		curr->ambient.r = rgb[0];
		curr->ambient.g = rgb[1];
		curr->ambient.b = rgb[2];
	} else if (lightColorChunkId_ == CHUNK_DIFFUSE) {
		curr->diffuse.r = rgb[0];
		curr->diffuse.g = rgb[1];
		curr->diffuse.b = rgb[2];
	} else if (lightColorChunkId_ == CHUNK_SPECULAR) {
		curr->specular.r = rgb[0];
		curr->specular.g = rgb[1];
		curr->specular.b = rgb[2];
	}
}

void ModelReader3DS::chunkRGBBReader(FILE* f, unsigned int indent, unsigned long p) {
	byte rgb[3];
	M3D_FREAD_EXT(rgb, f);	
    M3D_REPORT_CHUNK_DATA(indent, verbose,
		("Red: %d, Green: %d, Blue: %d", rgb[0], rgb[1], rgb[2]));

	// add color to material
	Ptr<Material> curr = currentMaterial();
	if (lightColorChunkId_ == CHUNK_AMBIENT) {
		curr->ambient.r = rgb[0] / 255.0f;
		curr->ambient.g = rgb[1] / 255.0f;
		curr->ambient.b = rgb[2] / 255.0f;
	} else if (lightColorChunkId_ == CHUNK_DIFFUSE) {
		curr->diffuse.r = rgb[0] / 255.0f;
		curr->diffuse.g = rgb[1] / 255.0f;
		curr->diffuse.b = rgb[2] / 255.0f;
	} else if (lightColorChunkId_ == CHUNK_SPECULAR) {
		curr->specular.r = rgb[0] / 255.0f;
		curr->specular.g = rgb[1] / 255.0f;
		curr->specular.b = rgb[2] / 255.0f;
	}
}

/*
void ModelReader3DS::chunkMeshVersionReader(FILE* f, unsigned int indent, unsigned long p) {
	word version;
	M3D_FREAD_EXT(version, f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",version));
}
*/

void ModelReader3DS::chunkMasterScaleReader(FILE* f, unsigned int indent, unsigned long p) {
	M3D_FREAD_EXT(masterScale_, f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %f", masterScale_));
}

void ModelReader3DS::chunkObjBlockReader(FILE* f, unsigned int indent, unsigned long p) {
	// create new object block
	objectBlocks_.push_back(new ObjectBlock);

	m3dFreadName(f);

	// add name to object block
	Ptr<ObjectBlock> curr = currentObjectBlock();
	string name(nameBuffer);
	curr->name = name;

	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Object name: \"%s\"", nameBuffer));
}

void ModelReader3DS::chunkVertListReader(FILE* f, unsigned int indent, unsigned long p) {
    word nv;

	// read number of vertexes
    M3D_FREAD_EXT(nv, f);

	vector<Vertex> vertexes(nv);

    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Vertices: %d", (int) nv));

    if (nv > MAX_VERTICES) {
		fprintf(stderr, "Object exceeded MAX_VERTICES count.\n");
		return; 
    }

	for (int i = 0; i < nv; i++) {
        M3D_FREAD_EXT(vertexes[i], f);
		M3D_REPORT_CHUNK_DATA(indent, verbose,
			("X: %f, Y: %f, Z: %f", vertexes[i].x, vertexes[i].y, vertexes[i].z));
    }
	
	// add vertexes to object block
	Ptr<ObjectBlock> curr = currentObjectBlock();
	curr->mesh.vertexes = vertexes;
}

void ModelReader3DS::chunkFaceListReader(FILE* f, unsigned int indent, unsigned long p) {
    word nv;
    word face[4];

	// read number of faces
    M3D_FREAD_EXT(nv,f);

	vector<Face> faces(nv);
	
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Faces: %d", (int)nv));

	for (int i = 0; i < nv; i++) {
        M3D_FREAD_EXT(face, f);
		faces[i].a = face[0];
		faces[i].b = face[1];
		faces[i].c = face[2];
		faces[i].flags = face[3];
        M3D_REPORT_CHUNK_DATA(indent, verbose, 
			("A: %d, B: %d, C: %d, Flags 0x%X", faces[i].a, faces[i].b, faces[i].c, faces[i].flags));
    }

	// add face vertexes to object block
	Ptr<ObjectBlock> curr = currentObjectBlock();
	curr->mesh.faces = faces;
}

void ModelReader3DS::chunkFaceMatReader(FILE* f, unsigned int indent, unsigned long p) {
	word nf, face;

	// get current mesh faces
	Ptr<ObjectBlock> curr = currentObjectBlock();
	vector<Face> &faces = curr->mesh.faces;

	m3dFreadName(f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Material name for faces: \"%s\"", nameBuffer));

    M3D_FREAD_EXT(nf, f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Faces with this material: %d", nf));
	
    for (int i = 0; i < nf; ++i) {
        M3D_FREAD_EXT(face,f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("Face: %d", face));

		// set face material
		string matName(nameBuffer);
		faces[face].materialName = matName;
    }
}

void ModelReader3DS::chunkMapListReader(FILE* f, unsigned int indent, unsigned long p) {
    word nv;

	// read number of mappings
    M3D_FREAD_EXT(nv, f);

	vector<Mapping> mappings(nv);

	for(int i = 0; i < nv; ++i) {
        M3D_FREAD_EXT(mappings[i], f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("U: %f, V: %f", mappings[i].u, mappings[i].v));
    }

	// add face vertexes to object block
	Ptr<ObjectBlock> curr = currentObjectBlock();
	curr->mesh.mappings = mappings;
}

/*
void ModelReader3DS::chunkSmooListReader(FILE* f, unsigned int indent, unsigned long p) {
    dword s;

    while (ftell(f) < p) {
        M3D_FREAD_EXT(s,f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("Smoothing groups: %lX",s));
        //for (i = 0; i < 32; i++)
        //    if (s & (1 << i))
        //        printf("%d, ", i + 1);
        //if(printf_3ds) printf("\n");
    }
}
*/
/*
void ModelReader3DS::chunkTrMatrixReader(FILE* f, unsigned int indent, unsigned long p) {
    float rot[9];
    float trans[3];
    M3D_FREAD_EXT(rot,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Rotation matrix:"));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("\t%f, %f, %f", rot[0], rot[1], rot[2]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("\t%f, %f, %f", rot[3], rot[4], rot[5]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("\t%f, %f, %f", rot[6], rot[7], rot[8]));
    M3D_FREAD_EXT(trans,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Translation matrix: %f, %f, %f", trans[0], trans[1], trans[2]));
}
*/
/*
void ModelReader3DS::chunkLightReader(FILE* f, unsigned int indent, unsigned long p) {
    float l[3];
    M3D_FREAD_EXT(l,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("X: %f, Y: %f, Z: %f", l[0], l[1], l[2]));
}
*/
/*
void ModelReader3DS::chunkSpotlightReader(FILE* f, unsigned int indent, unsigned long p) {
    float sl[5];
    M3D_FREAD_EXT(sl,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Target X: %f, Y: %f, Z: %f; Hotspot %f, Falloff %f", sl[0], sl[1], sl[2], sl[3], sl[4]));
}
*/
/*
void ModelReader3DS::chunkCameraReader(FILE* f, unsigned int indent, unsigned long p) {
    float c[8];
    M3D_FREAD_EXT(c,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Position: X: %f, Y: %f, Z: %f", c[0], c[1], c[2]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Target: X: %f, Y: %f, Z: %f", c[3], c[4], c[5]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Bank: %f, Lens: %f", c[6], c[7]));
}
*/

void ModelReader3DS::chunkMaterialReader(FILE* f, unsigned int indent, unsigned long p) {
	// create new material reader
	//materials_.push_back(new Material);
	currMaterial_ = new Material;
}

void ModelReader3DS::chunkMatNameReader(FILE* f, unsigned int indent, unsigned long p) {
	m3dFreadName(f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Material name \"%s\"", nameBuffer));

	// add name to material
	Ptr<Material> curr = currentMaterial();
	string name(nameBuffer);
	curr->name = name;
	// add material to lookup map
	materials_[name] = curr;
}

void ModelReader3DS::chunkAmbientReader(FILE* f, unsigned int indent, unsigned long p) {
	lightColorChunkId_ = CHUNK_AMBIENT;
}

void ModelReader3DS::chunkDiffuseReader(FILE* f, unsigned int indent, unsigned long p) {
	lightColorChunkId_ = CHUNK_DIFFUSE;
}

void ModelReader3DS::chunkSpecularReader(FILE* f, unsigned int indent, unsigned long p) {
	lightColorChunkId_ = CHUNK_SPECULAR;
}

void ModelReader3DS::chunkMapFileReader(FILE* f, unsigned int indent, unsigned long p) {
	m3dFreadName(f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Map filename \"%s\"", nameBuffer));

	// add map filename to material
	Ptr<Material> curr = currentMaterial();
	string mapFilename(nameBuffer);
	curr->mapFilename = mapFilename;
}

void ModelReader3DS::print() {
	cout << "Dumping model:" << endl;
	cout << "# Materials: " << materials_.size() << endl;
	cout << "# Object Blocks: " << objectBlocks_.size() << endl;
}

/*
void ModelReader3DS::chunkKfHdrReader(FILE* f, unsigned int indent, unsigned long p) {
	word revision; 
	word animlen; 

	M3D_FREAD_EXT(revision,f);
	m3dFreadName(f);
	M3D_FREAD_EXT(animlen,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose,
		("Name = %s, revision=%d, animlen=%d",nameBuffer,revision,animlen));
}
*/
/*
void ModelReader3DS::chunkFramesReader(FILE* f, unsigned int indent, unsigned long p) {
    dword frames[2];
    M3D_FREAD_EXT(frames,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose,
		("Start: %ld, End: %ld", frames[0], frames[1]));
}
*/
/*
void ModelReader3DS::chunkKfCurTimeReader(FILE* f, unsigned int indent, unsigned long p) {
	word curframe;
	M3D_FREAD_EXT(curframe,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",curframe));
}
*/
/*
void ModelReader3DS::chunkNodeIdReader(FILE* f, unsigned int indent, unsigned long p) {
	short id;
	M3D_FREAD_EXT(id,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",id));
}
*/
/*
void ModelReader3DS::chunkNodeHdrReader(FILE* f, unsigned int indent, unsigned long p) {
	word flags[2];
	word heirarchy;

	m3dFreadName(f);
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(heirarchy,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Name = %s, flags=(0x%X,0x%X), heirarchy=%d",nameBuffer,flags[0],flags[1],heirarchy));
}
*/
/*
void ModelReader3DS::chunkPivotReader(FILE* f, unsigned int indent, unsigned long p) {
	float pivot_x, pivot_y, pivot_z; 
	M3D_FREAD_EXT(pivot_x,f);
	M3D_FREAD_EXT(pivot_y,f);
	M3D_FREAD_EXT(pivot_z,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("x,y,z = %f,%f,%f",pivot_x,pivot_y,pivot_z));
}
*/
/*
void ModelReader3DS::chunkInstanceNameReader(FILE* f, unsigned int indent, unsigned long p) {
	m3dFreadName(f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Name = %s",nameBuffer));
}
*/
/*
void ModelReader3DS::chunkPosTrackTagReader(FILE* f, unsigned int indent, unsigned long p) {
	struct pos_track_tag_struct {
		word framenum;
		dword unknown;
		float pos_x, pos_y, pos_z; 
	}*  pos;
	word flags;
	word unknown[4];
	word keys;
	word unknown2;
	word i;
	
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(unknown,f);
	M3D_FREAD_EXT(keys,f);
	M3D_FREAD_EXT(unknown2,f);

	pos = malloc_(struct pos_track_tag_struct, keys);
	// TODO: assert out
	//assert(pos != NULL);

	/*
	 * DAMN PADDING MAKES NEXT LINE NOT WORK:
	 * if (keys != fread(pos, sizeof(struct pos_track_tag_struct), keys, f)) { m3dReportError(p); return; }
	 *
	
	for (i=0;i<keys;++i) {
		M3D_FREAD_EXT(pos[i].framenum, f);
		M3D_FREAD_EXT(pos[i].unknown, f);
		M3D_FREAD_EXT(pos[i].pos_x, f);
		M3D_FREAD_EXT(pos[i].pos_y, f);
		M3D_FREAD_EXT(pos[i].pos_z, f);
	}
		

	M3D_REPORT_CHUNK_DATA(indent, verbose, ("flags=%d, unknown=(0x%X,0x%X,0x%X,0x%X), keys=%d, unknown2=%d",flags,unknown[0],unknown[1],unknown[2],unknown[3],keys,unknown2));
	for (i=0;i<keys;++i) M3D_REPORT_CHUNK_DATA(indent, verbose, ("%d)\tframenum=%d, unknown=0x%X, pos=(%f,%f,%f)",
				i,pos[i].framenum,(unsigned int)pos[i].unknown,pos[i].pos_x,pos[i].pos_y,pos[i].pos_z));

	free(pos);
}
*/
/*
void ModelReader3DS::chunkRotTrackTagReader(FILE* f, unsigned int indent, unsigned long p) {
	struct rot_track_tag_struct {
		word framenum;
		dword unknown;
		float rotation_rad;
		float axis_x, axis_y, axis_z; 
	}*  rot;
	word flags;
	word unknown[4];
	word keys;
	word unknown2;
	word i;
	
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(unknown,f);
	M3D_FREAD_EXT(keys,f);
	M3D_FREAD_EXT(unknown2,f);

	rot = malloc_(struct rot_track_tag_struct, keys);
	// TODO: assert out
	//assert(rot != NULL);
	
	for (i=0;i<keys;++i) {
		M3D_FREAD_EXT(rot[i].framenum, f);
		M3D_FREAD_EXT(rot[i].unknown, f);
		M3D_FREAD_EXT(rot[i].rotation_rad, f);
		M3D_FREAD_EXT(rot[i].axis_x, f);
		M3D_FREAD_EXT(rot[i].axis_y, f);
		M3D_FREAD_EXT(rot[i].axis_z, f);
	}
		

	M3D_REPORT_CHUNK_DATA(indent, verbose, ("flags=%d, unknown=(0x%X,0x%X,0x%X,0x%X), keys=%d, unknown2=%d",flags,unknown[0],unknown[1],unknown[2],unknown[3],keys,unknown2));
	for (i=0;i<keys;++i) M3D_REPORT_CHUNK_DATA(indent, verbose, ("%d)\tframenum=%d, unknown=0x%X, rotation=%f, axis=(%f,%f,%f)",
				i,rot[i].framenum,(unsigned int)rot[i].unknown,rot[i].rotation_rad, rot[i].axis_x,rot[i].axis_y,rot[i].axis_z));

	free(rot);
}
*/
/*
void ModelReader3DS::chunkSclTrackTagReader(FILE* f, unsigned int indent, unsigned long p) {
	struct pos_track_tag_struct {
		word framenum;
		dword unknown;
		float scale_x, scale_y, scale_z; 
	}*  scl;
	word flags;
	word unknown[4];
	word keys;
	word unknown2;
	word i;
	
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(unknown,f);
	M3D_FREAD_EXT(keys,f);
	M3D_FREAD_EXT(unknown2,f);

	scl = malloc_(struct pos_track_tag_struct, keys);
	// TODO: assert out
	//assert(scl != NULL);	

	for (i=0;i<keys;++i) {
		M3D_FREAD_EXT(scl[i].framenum, f);
		M3D_FREAD_EXT(scl[i].unknown, f);
		M3D_FREAD_EXT(scl[i].scale_x, f);
		M3D_FREAD_EXT(scl[i].scale_y, f);
		M3D_FREAD_EXT(scl[i].scale_z, f);
	}
		

	M3D_REPORT_CHUNK_DATA(indent, verbose, ("flags=%d, unknown=(0x%X,0x%X,0x%X,0x%X), keys=%d, unknown2=%d",flags,unknown[0],unknown[1],unknown[2],unknown[3],keys,unknown2));
	for (i=0;i<keys;++i) M3D_REPORT_CHUNK_DATA(indent, verbose, ("%d)\tframenum=%d, unknown=0x%X, scale=(%f,%f,%f)",
				i,scl[i].framenum,(unsigned int)scl[i].unknown,scl[i].scale_x,scl[i].scale_y,scl[i].scale_z));

	free(scl);
}
*/
