#ifndef MODELREADER3DS_H
#define MODELREADER3DS_H

#include <stdio.h>
#include <string>
#include <vector>
#include <map>

#include "Ptr.h"
#include "PtrInterface.h"
#include "Reader3DS.h"

namespace M3DS {

	using namespace std;

	typedef struct {
		float x, y, z;
	} Vertex;

	typedef struct {
		float u, v;
	} Mapping;

	typedef struct {
		word a, b, c;
		word flags;
		string materialName;
	} Face;

	typedef struct {
		vector<Vertex> vertexes;
		vector<Mapping> mappings;
		vector<Face> faces;
	} Mesh;

	class ObjectBlock : public PtrInterface<ObjectBlock> {
	public:
		ObjectBlock() : name("") { }
		string name;
		Mesh mesh;
	};

	typedef struct {
		float r, g, b;
	} RGB;

	class Material : public PtrInterface<Material> {
	public:
		Material() : name(""), mapFilename("") { }
		string name;
		string mapFilename;
		RGB ambient;
		RGB diffuse;
		RGB specular;
	};

	class ModelReader3DS : public Reader3DS {
	public:
		ModelReader3DS() : 
		  Reader3DS(),
		  lightColorChunkId_(0),
		  masterScale_(1),
		  objectBlocks_(),
		  materials_() {
		}

		void print();
		vector<Ptr<ObjectBlock> > objectBlocks() { return objectBlocks_; }
		map<string, Ptr<Material> > materials() { return materials_; }

	protected:
		//void chunkM3dVersionReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkRGBFReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkRGBBReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkMeshVersionReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkMasterScaleReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkObjBlockReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkVertListReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkFaceListReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkFaceMatReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkMapListReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkSmooListReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkTrMatrixReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkLightReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkSpotlightReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkCameraReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkMaterialReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkMatNameReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkAmbientReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkDiffuseReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkSpecularReader(FILE* f, unsigned int indent, unsigned long p);
		void chunkMapFileReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkKfHdrReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkFramesReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkKfCurTimeReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkNodeIdReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkNodeHdrReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkPivotReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkInstanceNameReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkPosTrackTagReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkRotTrackTagReader(FILE* f, unsigned int indent, unsigned long p);
		//void chunkSclTrackTagReader(FILE* f, unsigned int indent, unsigned long p);

		Ptr<ObjectBlock> currentObjectBlock() { return objectBlocks_.back(); }
		Ptr<Material> currentMaterial() { return currMaterial_;	}

		word lightColorChunkId_;
		float masterScale_;

		vector<Ptr<ObjectBlock> > objectBlocks_;
		Ptr<Material> currMaterial_;
		map<string, Ptr<Material> > materials_;
	};

}

#endif