#include <SDL/SDL_opengl.h>

#include "Particle.h"

namespace GameEngine {

	Ptr<RandomNumberGenerator> RandomNumberGenerator::instance_ = NULL;

	//const float ParticleSystem::Z_INC = 0.00035;
	//int ParticleSystem::totalNumParticles_ = 0;
	//float ParticleSystem::zlevel_ = 0;

	void Particle::del() {
		sceneNode_->del();
	}

	void Particle::draw() {
		float viewMatrix[16];
		Ptr<Vec3> right, up, left, down;
		Ptr<Vec3> pos, tl, tr, bl, br;

		glPushMatrix();

		glDisable(GL_DEPTH_TEST);

		glEnable(GL_COLOR_MATERIAL);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

		glDisable(GL_LIGHTING);

		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);

		if (texture_) {
			texture_->bind();
		}

		glColor4f(color_->r, color_->g, color_->b, color_->a);

		if (billboardTrue_) {
			glGetFloatv(GL_MODELVIEW_MATRIX, viewMatrix);

			right = new Vec3(viewMatrix[0], viewMatrix[4], viewMatrix[8]);
			up = new Vec3(viewMatrix[1], viewMatrix[5], viewMatrix[9]);
			left = right->mult(-1);
			down = up->mult(-1);

			bl = left->plus(down); bl->normalize();
			br = right->plus(down); br->normalize();
			tr = right->plus(up); tr->normalize();
			tl = left->plus(up); tl->normalize();

			glBegin(GL_QUADS);

			glTexCoord2f(0.0, 0.0); glVertex3f(bl->x, bl->y, bl->z);
			glTexCoord2f(1.0, 0.0); glVertex3f(br->x, br->y, br->z);
			glTexCoord2f(1.0, 1.0); glVertex3f(tr->x, tr->y, tr->z);
			glTexCoord2f(0.0, 1.0); glVertex3f(tl->x, tl->y, tl->z);
		} else {
			glBegin(GL_QUADS);

			glTexCoord2f(0.0, 0.0); glVertex3f(-0.5, -0.5, 0.0);
			glTexCoord2f(1.0, 0.0); glVertex3f(0.5, -0.5, 0.0);
			glTexCoord2f(1.0, 1.0); glVertex3f(0.5, 0.5, 0.0);
			glTexCoord2f(0.0, 1.0); glVertex3f(-0.5, 0.5, 0.0);
		}

		glEnd();

		if (texture_) {
			texture_->unbind();
		}

		glEnable(GL_DEPTH_TEST);
		glEnable(GL_LIGHTING);
		glDisable(GL_BLEND);
		glDisable(GL_COLOR_MATERIAL);
		glPopMatrix();
	}

	void ParticleSystem::del() {
		for (int i = 0; i < particles_.size(); i++) {
			particles_[i]->del();
		}
		sceneNode_->del();
	}

	// Never called
	void ParticleSystem::draw() {
		vector<Ptr<Particle> >::iterator it = particles_.begin();
		while (it != particles_.end()) {
			Ptr<Particle> particle = *it;
			particle->draw();
			it++;
		}
	}

	void ParticleSystem::removeDeadParticles() {
		// remove dead particles
		int i = 0;
		while (i < particles_.size()) {
			Ptr<Particle> p = particles_[i];
			if (p->dead()) {
				particles_.erase(particles_.begin() + i);
				//totalNumParticles_--;
			} else {
				i++;
			}
		}
	}
}
