#include <iostream>
#include <SDL/SDL_opengl.h>
#include <vector>
#include <map>
#include <algorithm>

#include "Ptr.h"
#include "Picker.h"
#include "Engine.h"

namespace GameEngine {

	using namespace std;

	void Picker::startPicking(unsigned int clickX, unsigned int clickY) {
		int viewport[4];

		glSelectBuffer(PICK_BUFFER_SIZE, pickBuffer_);

		glGetIntegerv(GL_VIEWPORT, viewport);

		glRenderMode(GL_SELECT);

		glInitNames();

		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();

		gluPickMatrix(clickX, viewport[3] - clickY, 0.1, 0.1, viewport);
		gluPerspective(FOV_ANGLE, WINDOW_WIDTH/WINDOW_HEIGHT, CLIPPLANE_NEAR, 
			CLIPPLANE_FAR);

		glMatrixMode(GL_MODELVIEW);
	}

	void Picker::stopPicking() {
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glFlush();

		glMatrixMode(GL_MODELVIEW);

		int hits = glRenderMode(GL_RENDER);
		processHits(hits);
	}

	/**
	 * Here's an example of how the data is stored in the select buffer:
	 * Hit Record Contents 	Description
	 *	0 					No names have been stored for the first hit
	 *	4.2822e+009			Minimum depth for first hit
	 *	4.28436e+009 		Maximum depth for first hit
	 *	1 					Number of names for the second hit
	 *	4.2732e+009			Minimum depth for second hit
	 *	4.27334e+009 		Maximum depth for second hit
	 *	6 					A single name for the second hit
	 *	2 					Number of names for the third hit
	 *	4.27138e+009 		Minimum depth for third hit
	 *	4.27155e+009 		Maximum depth for third hit
	 *	2 					First name for third hit
	 *	5 					Second name for third hit
	 */
	void Picker::processHits (int hits) {
		int index = 0;
		unsigned int numNames, currZ, minZ, selectedName;

		index = 0;
		minZ = UINT_INITIAL;
		selectedName = UINT_INITIAL;

		for (int i = 0; i < hits; i++) {
			numNames = pickBuffer_[index];
			currZ = pickBuffer_[index + 1];
			if (currZ < minZ && numNames > 0) {
				minZ = currZ;
				selectedName = pickBuffer_[index + 3];
			}
			index += 3 + numNames;
		}

		if (selectedName < UINT_INITIAL) {
			pickedId_ = selectedName;
		} else {
			pickedId_ = 0;
		}
/*
		vector<unsigned int> ids;
		map<unsigned int, unsigned int> zmap;
		vector<unsigned int> zs;

		unsigned int numNames, currZ, minZ, selectedName;

		int index = 0;
		for (int i = 0; i < hits; i++) {
			numNames = pickBuffer_[index];
			currZ = pickBuffer_[index + 1];
			selectedName = pickBuffer_[index + 3];

			zmap[currZ] = selectedName;
			zs.push_back(currZ);

			index += 3 + numNames;			
		}

		sort(zs.begin(), zs.end());

		pickedIds_.clear();
		for (int i = 0; i < zs.size(); i++) {
			pickedIds_.push_back(zmap[zs[i]]);
			cout << pickedIds_[i] << " ";
		}
		cout << " size=" << pickedIds_.size() << endl;
*/
	}

}
