#include "Reader3DS.h"
#include <stdio.h>
#include <iostream>
#include <sstream>

using namespace M3DS;
using namespace std;

void Reader3DS::initChunkNames() {
	chunkNames[CHUNK_M3D_VERSION] =		"M3D Version";
	chunkNames[CHUNK_RGBF] =			"RGB float";
	chunkNames[CHUNK_RGBB] =			"RGB byte";
	chunkNames[CHUNK_MAIN] =			"Main";
	chunkNames[CHUNK_OBJMESH] =			"Object Mesh";
	chunkNames[CHUNK_MESH_VERSION] =	"Mesh Version";
	chunkNames[CHUNK_MASTER_SCALE] =	"Master Scale";
	chunkNames[CHUNK_BKGCOLOR] =		"Background color";
	chunkNames[CHUNK_AMBCOLOR] =		"Amb color";
	chunkNames[CHUNK_OBJBLOCK] =		"Object Block";
	chunkNames[CHUNK_TRIMESH] =			"Tri-Mesh";
	chunkNames[CHUNK_VERTLIST] =		"Vertex list";
	chunkNames[CHUNK_FACELIST] =		"Face list";
	chunkNames[CHUNK_FACEMAT] =			"Face material";
	chunkNames[CHUNK_MAPLIST] =			"Mappings list";
	chunkNames[CHUNK_SMOOLIST] =		"Smoothings";
	chunkNames[CHUNK_TRMATRIX] =		"Matrix";
	chunkNames[CHUNK_LIGHT] =			"Light";
	chunkNames[CHUNK_SPOTLIGHT] =		"Spotlight";
	chunkNames[CHUNK_CAMERA] =			"Camera";
	chunkNames[CHUNK_MATERIAL] =		"Material";
	chunkNames[CHUNK_MATNAME] =			"Material name";
	chunkNames[CHUNK_AMBIENT] =			"Ambient color";
	chunkNames[CHUNK_DIFFUSE] =			"Diffuse color";
	chunkNames[CHUNK_SPECULAR] =		"Specular color";
	chunkNames[CHUNK_TEXTURE] =			"Texture map";
	chunkNames[CHUNK_BUMPMAP] =			"Bump map";
	chunkNames[CHUNK_MAPFILE] =			"Map filename";
	chunkNames[CHUNK_KEYFRAMER] =		"Keyframer data";
	chunkNames[CHUNK_KFHDR] =			"Keyframer header";
	chunkNames[CHUNK_FRAMES] =			"Frames";
	chunkNames[CHUNK_KFCURTIME] =		"Current Keyframe";
	chunkNames[CHUNK_OBJECT_NODE_TAG] =	"Object Node Tag";
	chunkNames[CHUNK_CAMERA_NODE_TAG] =	"Camera Node Tag";
	chunkNames[CHUNK_TARGET_NODE_TAG] =	"Target Node Tag";
	chunkNames[CHUNK_NODE_ID] =			"Node ID";
	chunkNames[CHUNK_NODE_HDR] =		"Node Header";
	chunkNames[CHUNK_PIVOT] =			"Pivot";
	chunkNames[CHUNK_INSTANCE_NAME] =	"Instance Name";
	chunkNames[CHUNK_POS_TRACK_TAG] =	"Position Track Tag";
	chunkNames[CHUNK_ROT_TRACK_TAG] =	"Rotation Track Tag";
	chunkNames[CHUNK_SCL_TRACK_TAG] =	"Scale Track Tag";
	chunkNames[CHUNK_FOV_TRACK_TAG] =	"Camera Field Of View Track Tag";
	chunkNames[CHUNK_ROL_TRACK_TAG] =	"Camera Roll Track Tag";
	chunkNames[CHUNK_BOUNDBOX] =		"BoundBox";
}

/*******************************************************************************
 *
 * Core chunk reading functions
 *
 ******************************************************************************/
/* Read a C string (ASCIIZ) from file 'f' into global var 'nameBuffer'. */
/* Stops reading on null-terminator or end-of-file. */
/* Stops storing if length exceeds maximum (defined by NAME_BUFFER_SIZE) */
void Reader3DS::m3dFreadName(FILE* f) {
	unsigned c = 0;
	int i = 0;
	while ( (i = fgetc(f)) != EOF && i != (int)'\0') {
		if (c < NAME_BUFFER_SIZE) {
			nameBuffer[c++] = (char)i;
		}
	}
	nameBuffer[c]='\0';
}

void Reader3DS::runChunkFunc(word id, FILE* f, unsigned int indent, unsigned long p) {
	switch (id) {
		case CHUNK_M3D_VERSION: chunkM3dVersionReader(f, indent, p); break;
		case CHUNK_RGBF: chunkRGBFReader(f, indent, p); break;
		case CHUNK_RGBB: chunkRGBBReader(f, indent, p); break;
		case CHUNK_MAIN: chunkMainReader(f, indent, p); break;
		case CHUNK_OBJMESH:	chunkObjMeshReader(f, indent, p); break;
		case CHUNK_MESH_VERSION: chunkMeshVersionReader(f, indent, p); break;
		case CHUNK_MASTER_SCALE: chunkMasterScaleReader(f, indent, p); break;
		case CHUNK_BKGCOLOR: chunkBkgColorReader(f, indent, p); break;
		case CHUNK_AMBCOLOR: chunkAmbColorReader(f, indent, p); break;
		case CHUNK_OBJBLOCK: chunkObjBlockReader(f, indent, p); break;
		case CHUNK_TRIMESH: chunkTriMeshReader(f, indent, p); break;
		case CHUNK_VERTLIST: chunkVertListReader(f, indent, p); break;
		case CHUNK_FACELIST: chunkFaceListReader(f, indent, p); break;
		case CHUNK_FACEMAT: chunkFaceMatReader(f, indent, p); break;
		case CHUNK_MAPLIST: chunkMapListReader(f, indent, p); break;
		case CHUNK_SMOOLIST: chunkSmooListReader(f, indent, p); break;
		case CHUNK_TRMATRIX: chunkTrMatrixReader(f, indent, p); break;
		case CHUNK_LIGHT: chunkLightReader(f, indent, p); break;
		case CHUNK_SPOTLIGHT: chunkSpotlightReader(f, indent, p); break;
		case CHUNK_CAMERA: chunkCameraReader(f, indent, p); break;
		case CHUNK_MATERIAL: chunkMaterialReader(f, indent, p); break;
		case CHUNK_MATNAME: chunkMatNameReader(f, indent, p); break;
		case CHUNK_AMBIENT: chunkAmbientReader(f, indent, p); break;
		case CHUNK_DIFFUSE: chunkDiffuseReader(f, indent, p); break;
		case CHUNK_SPECULAR: chunkSpecularReader(f, indent, p); break;
		case CHUNK_TEXTURE: chunkTextureReader(f, indent, p); break;
		case CHUNK_BUMPMAP: chunkBumpMapReader(f, indent, p); break;
		case CHUNK_MAPFILE: chunkMapFileReader(f, indent, p); break;
		case CHUNK_KEYFRAMER: chunkKeyFramerReader(f, indent, p); break;
		case CHUNK_KFHDR: chunkKfHdrReader(f, indent, p); break;
		case CHUNK_FRAMES: chunkFramesReader(f, indent, p); break;
		case CHUNK_KFCURTIME: chunkKfCurTimeReader(f, indent, p); break;
		case CHUNK_OBJECT_NODE_TAG: chunkObjectNodeTagReader(f, indent, p); break;
		case CHUNK_CAMERA_NODE_TAG: chunkCameraNodeTagReader(f, indent, p); break;
		case CHUNK_TARGET_NODE_TAG: chunkTargetNodeTagReader(f, indent, p); break;
		case CHUNK_NODE_ID: chunkNodeIdReader(f, indent, p); break;
		case CHUNK_NODE_HDR: chunkNodeHdrReader(f, indent, p); break;
		case CHUNK_PIVOT: chunkPivotReader(f, indent, p); break;
		case CHUNK_INSTANCE_NAME: chunkInstanceNameReader(f, indent, p); break;
		case CHUNK_POS_TRACK_TAG: chunkPosTrackTagReader(f, indent, p); break;
		case CHUNK_ROT_TRACK_TAG: chunkRotTrackTagReader(f, indent, p); break;
		case CHUNK_SCL_TRACK_TAG: chunkSclTrackTagReader(f, indent, p); break;
		case CHUNK_FOV_TRACK_TAG: chunkFovTrackTagReader(f, indent, p); break;
		case CHUNK_ROL_TRACK_TAG: chunkRolTrackTagReader(f, indent, p); break;
		case CHUNK_BOUNDBOX: chunkBoundBoxReader(f, indent, p); break;
		case CHUNK_NULL: chunkNullReader(f, indent, p); break;
		default: chunkUnknownReader(f, indent, p); break;
	}

	// Recurse on children chunks only for certain chunk types
	switch (id) {
		case CHUNK_MAIN:
		case CHUNK_OBJMESH:
		case CHUNK_BKGCOLOR:
		case CHUNK_AMBCOLOR:
		case CHUNK_OBJBLOCK:
		case CHUNK_TRIMESH:
		case CHUNK_FACELIST:
		case CHUNK_LIGHT:
		case CHUNK_MATERIAL:
		case CHUNK_AMBIENT:
		case CHUNK_DIFFUSE:
		case CHUNK_SPECULAR:
		case CHUNK_TEXTURE:
		case CHUNK_BUMPMAP:
		case CHUNK_KEYFRAMER:
		case CHUNK_OBJECT_NODE_TAG:
		case CHUNK_CAMERA_NODE_TAG:
		case CHUNK_TARGET_NODE_TAG:
			// intentional fall through
			chunkReader(f, indent, p);
			break;
		default:
			break;
	}
}

string Reader3DS::chunkName(int id) {
	string name;
	if (chunkNames.find(id) == chunkNames.end()) {
		return name;
	}
	return chunkNames[id];
}

/* Read a chunk from f and call its appropriate handler */
void Reader3DS::chunkReader(FILE* f, unsigned int ind, unsigned long p) {
	struct chunk_header h;
	unsigned long pc;

	pc = ftell(f);
	while (pc < p) {

		/*if (fread(&h, sizeof(h), 1, f) != 1) return;*/
		M3D_FREAD_EXT(h,f);

		if (verbose) {
			string name = chunkName(h.id);
			if (!name.empty()) {
				printf("%*cChunk type: \"%s\", offset 0x%lX, size %ld bytes\n", 
					ind, ' ', name.c_str(), pc, h.len);
			} else {
				printf("%*cUnknown chunk: 0x%04X, offset 0x%lX, size: %ld bytes.\n", ind, ' ', h.id, pc, h.len);
			}
			//(void) fflush(stdout);
		}

		pc += h.len;

		ind += DUMP3DS_INDENT;

		runChunkFunc(h.id, f, ind, pc);

		ind -= DUMP3DS_INDENT;

		(void) fseek(f, pc, SEEK_SET);

		if (ferror(f)) {
			if(verbose) printf("Error occured!\n");
			break;
		}
	}
}


/* Start process a 3DS file ("filename") using handlers from handlers_list */
void Reader3DS::process3ds(string filename) {//, chunk_handler* handlers_list) {
	FILE* f;
	long size;

	if ((f = fopen(filename.c_str(), "rb")) == NULL ) {
		cout << "3dsrdr: Error opening " << filename << " for read. Quitting." << endl;
		return;
	}

    (void) fseek(f, 0, SEEK_END);
    size = ftell(f);
    (void) fseek(f, 0, SEEK_SET);

    chunkReader(f, 0, size);

    (void) fclose(f);
}

/*******************************************************************************
 *
 * Print chunk handlers
 *
 ******************************************************************************/
void PrintReader3DS::chunkM3dVersionReader(FILE* f, unsigned int indent, unsigned long p) {
	word version;
	M3D_FREAD_EXT(version,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",version));
}

void PrintReader3DS::chunkRGBFReader(FILE* f, unsigned int indent, unsigned long p) {
    float c[3];
	M3D_FREAD_EXT(c,f);	
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Red: %f, Green: %f, Blue: %f", c[0], c[1], c[2]));
}

void PrintReader3DS::chunkRGBBReader(FILE* f, unsigned int indent, unsigned long p) {
    byte c[3];
	M3D_FREAD_EXT(c,f);	
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Red: %d, Green: %d, Blue: %d", c[0], c[1], c[2]));
}

void PrintReader3DS::chunkMeshVersionReader(FILE* f, unsigned int indent, unsigned long p) {
	word version;
	M3D_FREAD_EXT(version,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",version));
}

void PrintReader3DS::chunkMasterScaleReader(FILE* f, unsigned int indent, unsigned long p) {
	float scale;
	M3D_FREAD_EXT(scale,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %f",scale));

}

void PrintReader3DS::chunkObjBlockReader(FILE* f, unsigned int indent, unsigned long p) {
	m3dFreadName(f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Object name: \"%s\"", nameBuffer));
}

void PrintReader3DS::chunkVertListReader(FILE* f, unsigned int indent, unsigned long p) {
    word nv;
	unsigned i;
    float v[3];

    M3D_FREAD_EXT(nv,f);

    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Vertices: %d", (int)nv));

    if(nv>MAX_VERTICES) {
		fprintf(stderr, "Object exceeded MAX_VERTICES count.\n");
		return; 
    }

	for(i=0;i<nv;i++) {
        M3D_FREAD_EXT(v,f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("X: %f, Y: %f, Z: %f", v[0], v[1], v[2]));
    }
	
}

void PrintReader3DS::chunkFaceListReader(FILE* f, unsigned int indent, unsigned long p) {
    word nv;
    word face[4];

    M3D_FREAD_EXT(nv,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Faces: %d", (int)nv));

    while (nv-- > 0) {
        M3D_FREAD_EXT(face,f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("A: %d, B: %d, C: %d, Flags 0x%X", face[0], face[1], face[2], face[3]));
    }
}

void PrintReader3DS::chunkFaceMatReader(FILE* f, unsigned int indent, unsigned long p) {
	word nf, face;
	unsigned i;

	m3dFreadName(f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Material name for faces: \"%s\"", nameBuffer));

    M3D_FREAD_EXT(nf,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Faces with this material: %d", nf));
	
    for(i=0;i<nf;++i) {
        M3D_FREAD_EXT(face,f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("Face: %d", face));
    }
}

void PrintReader3DS::chunkMapListReader(FILE* f, unsigned int indent, unsigned long p) {
    word nv;
    float uv[2];
	unsigned i;

    M3D_FREAD_EXT(nv,f);

	for(i=0;i<nv;++i) {
        M3D_FREAD_EXT(uv,f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("U: %f, V: %f", uv[0], uv[1]));
    }
	
}

void PrintReader3DS::chunkSmooListReader(FILE* f, unsigned int indent, unsigned long p) {
    dword s;
    /*int i;*/

    while (ftell(f) < p) {
        M3D_FREAD_EXT(s,f);
        M3D_REPORT_CHUNK_DATA(indent, verbose, ("Smoothing groups: %lX",s));
        /*for (i = 0; i < 32; i++)
            if (s & (1 << i))
                printf("%d, ", i + 1);
        if(printf_3ds) printf("\n");*/
    }
}

void PrintReader3DS::chunkTrMatrixReader(FILE* f, unsigned int indent, unsigned long p) {
    float rot[9];
    float trans[3];
    M3D_FREAD_EXT(rot,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Rotation matrix:"));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("\t%f, %f, %f", rot[0], rot[1], rot[2]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("\t%f, %f, %f", rot[3], rot[4], rot[5]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("\t%f, %f, %f", rot[6], rot[7], rot[8]));
    M3D_FREAD_EXT(trans,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Translation matrix: %f, %f, %f", trans[0], trans[1], trans[2]));

}

void PrintReader3DS::chunkLightReader(FILE* f, unsigned int indent, unsigned long p) {
    float l[3];
    M3D_FREAD_EXT(l,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("X: %f, Y: %f, Z: %f", l[0], l[1], l[2]));
}

void PrintReader3DS::chunkSpotlightReader(FILE* f, unsigned int indent, unsigned long p) {
    float sl[5];
    M3D_FREAD_EXT(sl,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Target X: %f, Y: %f, Z: %f; Hotspot %f, Falloff %f", sl[0], sl[1], sl[2], sl[3], sl[4]));
}
 
void PrintReader3DS::chunkCameraReader(FILE* f, unsigned int indent, unsigned long p) {
    float c[8];
    M3D_FREAD_EXT(c,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Position: X: %f, Y: %f, Z: %f", c[0], c[1], c[2]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Target: X: %f, Y: %f, Z: %f", c[3], c[4], c[5]));
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Bank: %f, Lens: %f", c[6], c[7]));
}

void PrintReader3DS::chunkMatNameReader(FILE* f, unsigned int indent, unsigned long p) {
	m3dFreadName(f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Material name \"%s\"", nameBuffer));
}

void PrintReader3DS::chunkMapFileReader(FILE* f, unsigned int indent, unsigned long p) {
	m3dFreadName(f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Map filename \"%s\"", nameBuffer));
}

void PrintReader3DS::chunkKfHdrReader(FILE* f, unsigned int indent, unsigned long p) {
	word revision; 
	word animlen; 

	M3D_FREAD_EXT(revision,f);
	m3dFreadName(f);
	M3D_FREAD_EXT(animlen,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Name = %s, revision=%d, animlen=%d",nameBuffer,revision,animlen));
}

void PrintReader3DS::chunkFramesReader(FILE* f, unsigned int indent, unsigned long p) {
    dword frames[2];
    M3D_FREAD_EXT(frames,f);
    M3D_REPORT_CHUNK_DATA(indent, verbose, ("Start: %ld, End: %ld", frames[0], frames[1]));
}

void PrintReader3DS::chunkKfCurTimeReader(FILE* f, unsigned int indent, unsigned long p) {
	word curframe;
	M3D_FREAD_EXT(curframe,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",curframe));
}

void PrintReader3DS::chunkNodeIdReader(FILE* f, unsigned int indent, unsigned long p) {
	short id;
	M3D_FREAD_EXT(id,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("= %d",id));
}

void PrintReader3DS::chunkNodeHdrReader(FILE* f, unsigned int indent, unsigned long p) {
	word flags[2];
	word heirarchy;

	m3dFreadName(f);
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(heirarchy,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Name = %s, flags=(0x%X,0x%X), heirarchy=%d",nameBuffer,flags[0],flags[1],heirarchy));
}

void PrintReader3DS::chunkPivotReader(FILE* f, unsigned int indent, unsigned long p) {
	float pivot_x, pivot_y, pivot_z; 
	M3D_FREAD_EXT(pivot_x,f);
	M3D_FREAD_EXT(pivot_y,f);
	M3D_FREAD_EXT(pivot_z,f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("x,y,z = %f,%f,%f",pivot_x,pivot_y,pivot_z));
}

void PrintReader3DS::chunkInstanceNameReader(FILE* f, unsigned int indent, unsigned long p) {
	m3dFreadName(f);
	M3D_REPORT_CHUNK_DATA(indent, verbose, ("Name = %s",nameBuffer));
}

void PrintReader3DS::chunkPosTrackTagReader(FILE* f, unsigned int indent, unsigned long p) {
	struct pos_track_tag_struct {
		word framenum;
		dword unknown;
		float pos_x, pos_y, pos_z; 
	}*  pos;
	word flags;
	word unknown[4];
	word keys;
	word unknown2;
	word i;
	
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(unknown,f);
	M3D_FREAD_EXT(keys,f);
	M3D_FREAD_EXT(unknown2,f);

	pos = malloc_(struct pos_track_tag_struct, keys);
	// TODO: assert out
	//assert(pos != NULL);

	/*
	 * DAMN PADDING MAKES NEXT LINE NOT WORK:
	 * if (keys != fread(pos, sizeof(struct pos_track_tag_struct), keys, f)) { m3dReportError(p); return; }
	 */
	
	for (i=0;i<keys;++i) {
		M3D_FREAD_EXT(pos[i].framenum, f);
		M3D_FREAD_EXT(pos[i].unknown, f);
		M3D_FREAD_EXT(pos[i].pos_x, f);
		M3D_FREAD_EXT(pos[i].pos_y, f);
		M3D_FREAD_EXT(pos[i].pos_z, f);
	}
		

	M3D_REPORT_CHUNK_DATA(indent, verbose, ("flags=%d, unknown=(0x%X,0x%X,0x%X,0x%X), keys=%d, unknown2=%d",flags,unknown[0],unknown[1],unknown[2],unknown[3],keys,unknown2));
	for (i=0;i<keys;++i) M3D_REPORT_CHUNK_DATA(indent, verbose, ("%d)\tframenum=%d, unknown=0x%X, pos=(%f,%f,%f)",
				i,pos[i].framenum,(unsigned int)pos[i].unknown,pos[i].pos_x,pos[i].pos_y,pos[i].pos_z));

	free(pos);
}

void PrintReader3DS::chunkRotTrackTagReader(FILE* f, unsigned int indent, unsigned long p) {
	struct rot_track_tag_struct {
		word framenum;
		dword unknown;
		float rotation_rad;
		float axis_x, axis_y, axis_z; 
	}*  rot;
	word flags;
	word unknown[4];
	word keys;
	word unknown2;
	word i;
	
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(unknown,f);
	M3D_FREAD_EXT(keys,f);
	M3D_FREAD_EXT(unknown2,f);

	rot = malloc_(struct rot_track_tag_struct, keys);
	// TODO: assert out
	//assert(rot != NULL);
	
	for (i=0;i<keys;++i) {
		M3D_FREAD_EXT(rot[i].framenum, f);
		M3D_FREAD_EXT(rot[i].unknown, f);
		M3D_FREAD_EXT(rot[i].rotation_rad, f);
		M3D_FREAD_EXT(rot[i].axis_x, f);
		M3D_FREAD_EXT(rot[i].axis_y, f);
		M3D_FREAD_EXT(rot[i].axis_z, f);
	}
		

	M3D_REPORT_CHUNK_DATA(indent, verbose, ("flags=%d, unknown=(0x%X,0x%X,0x%X,0x%X), keys=%d, unknown2=%d",flags,unknown[0],unknown[1],unknown[2],unknown[3],keys,unknown2));
	for (i=0;i<keys;++i) M3D_REPORT_CHUNK_DATA(indent, verbose, ("%d)\tframenum=%d, unknown=0x%X, rotation=%f, axis=(%f,%f,%f)",
				i,rot[i].framenum,(unsigned int)rot[i].unknown,rot[i].rotation_rad, rot[i].axis_x,rot[i].axis_y,rot[i].axis_z));

	free(rot);
}

void PrintReader3DS::chunkSclTrackTagReader(FILE* f, unsigned int indent, unsigned long p) {
	struct pos_track_tag_struct {
		word framenum;
		dword unknown;
		float scale_x, scale_y, scale_z; 
	}*  scl;
	word flags;
	word unknown[4];
	word keys;
	word unknown2;
	word i;
	
	M3D_FREAD_EXT(flags,f);
	M3D_FREAD_EXT(unknown,f);
	M3D_FREAD_EXT(keys,f);
	M3D_FREAD_EXT(unknown2,f);

	scl = malloc_(struct pos_track_tag_struct, keys);
	// TODO: assert out
	//assert(scl != NULL);	

	for (i=0;i<keys;++i) {
		M3D_FREAD_EXT(scl[i].framenum, f);
		M3D_FREAD_EXT(scl[i].unknown, f);
		M3D_FREAD_EXT(scl[i].scale_x, f);
		M3D_FREAD_EXT(scl[i].scale_y, f);
		M3D_FREAD_EXT(scl[i].scale_z, f);
	}
		

	M3D_REPORT_CHUNK_DATA(indent, verbose, ("flags=%d, unknown=(0x%X,0x%X,0x%X,0x%X), keys=%d, unknown2=%d",flags,unknown[0],unknown[1],unknown[2],unknown[3],keys,unknown2));
	for (i=0;i<keys;++i) M3D_REPORT_CHUNK_DATA(indent, verbose, ("%d)\tframenum=%d, unknown=0x%X, scale=(%f,%f,%f)",
				i,scl[i].framenum,(unsigned int)scl[i].unknown,scl[i].scale_x,scl[i].scale_y,scl[i].scale_z));

	free(scl);
}
