#include <SDL/SDL_opengl.h>

#include "Ptr.h"
#include "SceneNode.h"

namespace GameEngine {

	using namespace std;

	void SceneNode::render() {
		glPushMatrix();
		applyTransform();
		renderChildren();
		glPopMatrix();
	}

	void SceneNode::renderChildren() {
		vector<Ptr<SceneNode> >::iterator childIter = children_.begin();
		while (childIter != children_.end()) {
			Ptr<SceneNode> child = *childIter;
			child->render();
			childIter++;
		}
	}

	void SceneNode::applyTransform() {
		transformMatrix_->apply();
	}

	void SceneNode::moveTo(float x, float y, float z) {
		Ptr<Vertex3> m = new Vertex3(x, y, z);
		Ptr<Vertex3> pos = worldPosition();
		Ptr<Vec3> trans = m->sub(pos);
		translate(trans->x, trans->y, trans->z);
	}

	void SceneNode::translate(float x, float y, float z) {
		transformMatrix_->translate(x, y, z);
	}

	void SceneNode::scale(float x, float y, float z) {
		transformMatrix_->scale(x, y, z);
	}

	void SceneNode::rotate(float angle, float x, float y, float z) {
		transformMatrix_->rotate(angle, x, y, z);
	}

	void SceneNode::childNew(Ptr<SceneNode> child) {
		child->parentIs(this);
		children_.push_back(child);
	}

	void SceneNode::childDel(unsigned int id) {
		vector<Ptr<SceneNode> >::iterator childIter = children_.begin();
		while (childIter != children_.end()) {
			Ptr<SceneNode> child = *childIter;
			if (child->id() == id) {
				children_.erase(childIter);
				break;
			}
			childIter++;
		}
	}

	void SceneNode::childDel(Ptr<SceneNode> child) {
		return childDel(child->id());
	}

	void SceneNode::del() {
		if (parent_ != NULL) {
			parent_->childDel(this);
		}
	}

	Ptr<Matrix> SceneNode::worldTransform() {
		Ptr<Matrix> m = new Matrix;
/*		if (parent_) {
			m = parent_->worldTransform();
		} else {
			m = new Matrix;
		}
*/		// post-multiply like OpenGL
		return m->matrixMult(transformMatrix_);
	}

	Ptr<Vertex3> SceneNode::worldPosition() {
		Ptr<Vertex3> pos = new Vertex3;
		pos->x = transformMatrix_->value(12);
		pos->y = transformMatrix_->value(13);
		pos->z = transformMatrix_->value(14);
		return pos;
	}
}