#ifndef SCENENODE_H
#define SCENENODE_H

#include <string>
#include <vector>

#include "Geometry.h"
#include "UniqueEntity.h"
#include "Ptr.h"
#include "PtrInterface.h"

namespace GameEngine {

	using namespace Fwk;
	using namespace std;

	class SceneNode : public UniqueEntity {
	public:
		SceneNode() : UniqueEntity(), children_() {
			parent_ = NULL;
			transformMatrix_ = new Matrix;
		}
		SceneNode(string name) : UniqueEntity(name), children_() {
			parent_ = NULL;
			transformMatrix_ = new Matrix;
		}

		virtual void render();

		void moveTo(float x, float y, float z);
		void translate(float x, float y, float z);
		void rotate(float angle, float x, float y, float z);
		void scale(float x, float y, float z);

		void childNew(Ptr<SceneNode> child);
		void childDel(unsigned int id);
		void childDel(Ptr<SceneNode> child);
		void del();

		virtual void applyTransform();

		Ptr<SceneNode> parent() { return parent_; }
		void parentIs(Ptr<SceneNode> parent) { parent_ = parent; }

		Ptr<Vertex3> worldPosition();

		Ptr<Matrix> worldTransform();
		//Ptr<Matrix> transformMatrix() { return transformMatrix_; }

	protected:
		void renderChildren();

		Ptr<Matrix> transformMatrix_;
		Ptr<SceneNode> parent_;
		vector<Ptr<SceneNode> > children_;
	};

}

#endif