#include <GLee/GLee.h>
#include <SDL/SDL_opengl.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include "Ptr.h"
#include "Shader.h"

namespace GameEngine {

	void Shader::bind() {
		glUseProgramObjectARB(program_);
	}

	void Shader::unbind() {
		glUseProgramObjectARB(0);
	}

	void Shader::initShader() {
		// make the vertex shader
		string file = readFile(vertShaderFile_);
		char* vertsource = (char*) calloc(strlen(file.c_str()) + 1, sizeof(char));
		strcpy(vertsource, file.c_str());
		vertShader_ = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
		glShaderSourceARB(vertShader_, 1, (const GLcharARB **) &vertsource, NULL);
		free(vertsource);
		glCompileShaderARB(vertShader_);
		checkShaderLog(vertShader_);

		// make the fragment shader
		file = readFile(fragShaderFile_);
		char* fragsource = (char*) calloc(strlen(file.c_str()) + 1, sizeof(char));
		strcpy(fragsource, file.c_str());
		fragShader_ = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
		glShaderSourceARB(fragShader_, 1, (const GLcharARB **)&fragsource, NULL);
		free(fragsource);
		glCompileShaderARB(fragShader_);
		checkShaderLog(fragShader_);

		// link them into a program
		program_ = glCreateProgramObjectARB();
		glAttachObjectARB(program_, vertShader_);   
		glAttachObjectARB(program_, fragShader_);
		glLinkProgramARB(program_);
		checkShaderLog(program_);
	}

	string Shader::readFile(string filename) {
		ifstream myfile(filename.c_str());
		stringstream ss;
		if (myfile.is_open()) {
			string line;
			while (!myfile.eof()) {
				getline (myfile,line);
				ss << line << endl;
			}
			myfile.close();
		}
		return ss.str();
	}

	void Shader::checkShaderLog(GLuint shaderId) {
		char log[4096];
		int len;
		glGetInfoLogARB(shaderId, 4096, &len, log);
		printf("Shader compilation log:\n");
		printf(log);
		printf("\n");
		fflush(stdout);

		if (strlen(log)) return;
	}

}