#ifndef TIMER_H
#define TIMER_H

#include <SDL/SDL.h>

#include "Ptr.h"
#include "PtrInterface.h"

namespace GameEngine {

	using namespace Fwk;

	class Timer : public PtrInterface<Timer> {
	public:
		Timer() { }

		void start() {
			running_ = true;
			startTicks_ = SDL_GetTicks();
			lastTick_ = startTicks_;
		}

		virtual void update() {
			lastTick_ = SDL_GetTicks();
		}

		void stop() {
			running_ = false;
		}

		void setTicks(unsigned int ticks) { startTicks_ = lastTick_ = ticks; }

		unsigned int elapsedMillis() {
			return SDL_GetTicks() - lastTick_;
		}

		unsigned int totalMillis() {
			return SDL_GetTicks() - startTicks_;
		}

	protected:
		bool running_;
		unsigned int startTicks_;
		unsigned int lastTick_;
	};

	class FPS : public Timer {
	public:
		FPS() : 
			Timer(),
			fps_(0),
			frames_(0),
			prevSecond_(startTicks_ / 1000.0f) { }

		int fps() { return fps_; }

		void update() {
			Timer::update();
			frames_++;
			int currSecond = lastTick_ / 1000.0f;
			if (currSecond - prevSecond_ != 0) {
				prevSecond_ = currSecond;
				fps_ = frames_;
				frames_ = 0;
			}
		}

		void delayToRegulateFPS(int fps) {
			int delay = 1000.0f / fps - elapsedMillis();
			if (delay > 0) {
				SDL_Delay(delay);
			}
		}

	protected:
		int fps_;
		int frames_;
		int prevSecond_;
	};
}

#endif