#include <vector>
#include <math.h>
#include <GLee/GLee.h>
#include <SDL/SDL_opengl.h>
#include <GL/glut.h>
#include <iostream>

#include "Ptr.h"
#include "Tower.h"
#include "TextureManager.h"
#include "TowerColor.h"

namespace Tower {

	using namespace std;

	const float TowerData::DEFAULT_COST = 10;
	const float TowerData::DEFAULT_RANGE = 3;
	const float TowerData::DEFAULT_DAMAGE = 1;
	const float TowerData::DEFAULT_FIRING_DELAY = 1000;
	const int TowerData::FIRE_L1_COST = 10;
	const int TowerData::FIRE_L2_COST = 10;
	const int TowerData::FIRE_L3_COST = 10;
	const int TowerData::FIRE_L1_DAMAGE = 2;
	const int TowerData::FIRE_L2_DAMAGE = 2;
	const int TowerData::FIRE_L3_DAMAGE = 2;
	const int TowerData::FIRE_L1_RANGE = 1;
	const int TowerData::FIRE_L2_RANGE = 1;
	const int TowerData::FIRE_L3_RANGE = 1;
	const int TowerData::ICE_L1_COST = 10;
	const int TowerData::ICE_L2_COST = 10;
	const int TowerData::ICE_L3_COST = 10;
	const int TowerData::ICE_L1_DAMAGE = 2;
	const int TowerData::ICE_L2_DAMAGE = 2;
	const int TowerData::ICE_L3_DAMAGE = 2;
	const int TowerData::ICE_L1_RANGE = 1;
	const int TowerData::ICE_L2_RANGE = 1;
	const int TowerData::ICE_L3_RANGE = 1;
	const int TowerData::EARTH_L1_COST = 10;
	const int TowerData::EARTH_L2_COST = 10;
	const int TowerData::EARTH_L3_COST = 10;
	const int TowerData::EARTH_L1_DAMAGE = 2;
	const int TowerData::EARTH_L2_DAMAGE = 2;
	const int TowerData::EARTH_L3_DAMAGE = 2;
	const int TowerData::EARTH_L1_RANGE = 1;
	const int TowerData::EARTH_L2_RANGE = 1;
	const int TowerData::EARTH_L3_RANGE = 1;
	const float EnemyData::DEFAULT_GOLD = 5;
	const float EnemyData::DEFAULT_SPEED = 0.002;
	const float EnemyData::DEFAULT_HEALTH = 5;
	const float Projectile::DEFAULT_RADIUS = 0.125;
	const float Projectile::DEFAULT_SPEED = 0.005;
	const float Enemy::DEFAULT_RADIUS = 0.5;

	//
	// TowerUpgradeData
	//

	Ptr<TowerUpgradeData> TowerUpgradeData::copy() {
		Ptr<TowerUpgradeData> copy = new TowerUpgradeData();

		copy->cost_ = cost_;
		copy->range_ = range_;
		copy->damage_ = damage_;

		return copy;
	}

	//
	// TowerData
	//

	void TowerData::initUpgrades(TowerType towerType) {
		Ptr<TowerUpgradeData> l1, l2, l3;

		if (towerType == FireTowerType) {

			l1 = new TowerUpgradeData;
			l1->costIs(FIRE_L1_COST);
			l1->damageIs(FIRE_L1_DAMAGE);
			l1->rangeIs(FIRE_L1_RANGE);

			l2 = new TowerUpgradeData;
			l2->costIs(FIRE_L2_COST);
			l2->damageIs(FIRE_L2_DAMAGE);
			l2->rangeIs(FIRE_L2_RANGE);

			l3 = new TowerUpgradeData;
			l3->costIs(FIRE_L3_COST);
			l3->damageIs(FIRE_L3_DAMAGE);
			l3->rangeIs(FIRE_L3_RANGE);

			upgrades_.resize(3);
			upgrades_[0] = l1;
			upgrades_[1] = l2;
			upgrades_[2] = l3;

		} else if (towerType == IceTowerType) {

			l1 = new TowerUpgradeData;
			l1->costIs(ICE_L1_COST);
			l1->damageIs(ICE_L1_DAMAGE);
			l1->rangeIs(ICE_L1_RANGE);

			l2 = new TowerUpgradeData;
			l2->costIs(ICE_L2_COST);
			l2->damageIs(ICE_L2_DAMAGE);
			l2->rangeIs(ICE_L2_RANGE);

			l3 = new TowerUpgradeData;
			l3->costIs(ICE_L3_COST);
			l3->damageIs(ICE_L3_DAMAGE);
			l3->rangeIs(ICE_L3_RANGE);

			upgrades_.resize(3);
			upgrades_[0] = l1;
			upgrades_[1] = l2;
			upgrades_[2] = l3;

		} else if (towerType == EarthTowerType) {

			l1 = new TowerUpgradeData;
			l1->costIs(EARTH_L1_COST);
			l1->damageIs(EARTH_L1_DAMAGE);
			l1->rangeIs(EARTH_L1_RANGE);

			l2 = new TowerUpgradeData;
			l2->costIs(EARTH_L2_COST);
			l2->damageIs(EARTH_L2_DAMAGE);
			l2->rangeIs(EARTH_L2_RANGE);

			l3 = new TowerUpgradeData;
			l3->costIs(EARTH_L3_COST);
			l3->damageIs(EARTH_L3_DAMAGE);
			l3->rangeIs(EARTH_L3_RANGE);

			upgrades_.resize(3);
			upgrades_[0] = l1;
			upgrades_[1] = l2;
			upgrades_[2] = l3;

		}
	}

	void TowerData::upgrade() {
		if (upgrades_.size() >= upgradeLevel_ + 1) {
			Ptr<TowerUpgradeData> upgradeData = upgrades_[upgradeLevel_];
			range_ += upgradeData->range();
			damage_ += upgradeData->damage();
			upgradeLevel_++;
		}
	}

	Ptr<TowerData> TowerData::copy() {
		Ptr<TowerData> copy = new TowerData(towerType());

		copy->cost_ = cost_;
		copy->range_ = range_;
		copy->damage_ = damage_;
		copy->maxFireDelay_ = maxFireDelay_;
		copy->hudImage_ = hudImage_;
		copy->upgradeLevel_ = upgradeLevel_;

		copy->upgrades_ = upgrades_;
		/*
		vector<Ptr<TowerUpgradeData> >::iterator it = upgrades_.begin();
		while (it != upgrades_.end()) {
			// TODO: will upgrade ever be modified? if not, don't need deep copy here
			copy->upgradeNew((*it)->copy());
			it++;
		}
		*/

		return copy;
	}

	//
	// EnemyData
	//

	Ptr<EnemyData> EnemyData::copy() {
		Ptr<EnemyData> copy = new EnemyData(enemyType());
		
		copy->health_ = health_;
		copy->speed_ = speed_;

		return copy;
	}

	//
	// Tower
	//

	void Tower::del() {
		sceneNode_->del();
	}

	Ptr<Tower> Tower::copy() {
		Ptr<Tower> copy = new Tower(towerData_->towerType());

		copy->towerData_ = towerData_->copy();
		// TODO: copy scene node

		return copy;
	}

	void Tower::draw() {
		glEnable(GL_COLOR_MATERIAL);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

		Ptr<Color4> c = new Color4;
		if (towerData_->towerType() == TowerData::FireTowerType) {
			c = TowerColor::Fire();
		} else if (towerData_->towerType() == TowerData::IceTowerType) {
			c = TowerColor::Ice();
		} else if (towerData_->towerType() == TowerData::EarthTowerType) {
			c = TowerColor::Earth();
		}
		glColor4f(c->r, c->g, c->b, c->a);

		glPushName(id_);

		glutSolidCone(0.5, 1.5, 20, 20);

		glPopName();

		glDisable(GL_COLOR_MATERIAL);
	}

	Ptr<Vertex3> Tower::position() {
		return sceneNode_->worldPosition();
	}

	//
	// Projectile
	//

	Ptr<Projectile> Projectile::copy() {
		Ptr<Projectile> copy = new Projectile(tower_);
		
		copy->damage_ = damage_;
		copy->dead_ = dead_;
		copy->target_ = target_;
		if (velocity_) {
			Ptr<Vec3> vel = new Vec3(velocity_->x, velocity_->y, velocity_->z);
			copy->velocity_ = vel;
		}
		// TODO: copy scene node

		return copy;
	}

	void Projectile::del() {
		sceneNode_->del();
	}

	void Projectile::draw() {
		glEnable(GL_COLOR_MATERIAL);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

		Ptr<Color4> c = new Color4;
		if (projectileType() == TowerData::FireTowerType) {
			c = TowerColor::Fire();
		} else if (projectileType() == TowerData::IceTowerType) {
			c = TowerColor::Ice();
		} else if (projectileType() == TowerData::EarthTowerType) {
			c = TowerColor::Earth();
		}
		glColor4f(c->r, c->g, c->b, c->a);

		glutSolidSphere(DEFAULT_RADIUS, 20, 20);

		glDisable(GL_COLOR_MATERIAL);

		glPopName();
	}

	void Projectile::updatePosition(float elapsedMillis) {
		seekTarget();
		Ptr<Vec3> distVec = velocity_->mult(elapsedMillis);
		sceneNode_->translate(distVec->x, distVec->y, distVec->z);
	}

	Ptr<Vertex3> Projectile::position() {
		return sceneNode_->worldPosition();
	}

	bool Projectile::intersectsTarget() {
		Ptr<Vertex3> projPos = position();
		Ptr<Vertex3> enemyPos = target_->position();
		Ptr<Vec3> distVec = projPos->sub(enemyPos);
		float dist = distVec->mag();

		return dist < radius() + target_->radius();
	}

	void Projectile::seekTarget() {
		Ptr<Vertex3> pPos = position();
		Ptr<Vertex3> ePos = target_->position();
		Ptr<Vec3> vec = ePos->sub(pPos);
		vec->normalize();
		vec = vec->mult(DEFAULT_SPEED);
		velocityIs(vec);
	}

	//
	// Enemy
	//

	Ptr<Enemy> Enemy::copy() {
		Ptr<Enemy> copy = new Enemy(enemyData_->enemyType());

		copy->enemyData_ = enemyData_->copy();
		Ptr<Vec3> vel = new Vec3(velocity_->x, velocity_->y, velocity_->z);
		copy->velocity_ = vel;
		// TODO: copy scene node
		copy->material_ = material_;

		return copy;
	}

	void Enemy::del() {
		sceneNode_->del();
	}

	void Enemy::initMaterial() {
		material_ = new Material;
		Ptr<Texture> t;
		if (enemyData_->enemyType() == EnemyData::CalEnemyType) {
			t = TextureManager::Instance()->texture(TextureManager::CAL_IMAGE_FILE);
		} else if (enemyData_->enemyType() == EnemyData::UCLAEnemyType) {
			t = TextureManager::Instance()->texture(TextureManager::UCLA_IMAGE_FILE);
		} else if (enemyData_->enemyType() == EnemyData::USCEnemyType) {
			t = TextureManager::Instance()->texture(TextureManager::USC_IMAGE_FILE);
		}
		material_->texture = t;
	}

	void Enemy::draw() {
		if (material_) {
			glActiveTextureARB(GL_TEXTURE0_ARB);
			material_->bind();
		}

		GLUquadric *sphere = gluNewQuadric();
		gluQuadricTexture(sphere, true);
		gluSphere(sphere, DEFAULT_RADIUS, 20, 20);
		gluDeleteQuadric(sphere);

		if (material_) {
			glActiveTextureARB(GL_TEXTURE0_ARB);
			material_->unbind();
		}
	}

	void Enemy::updatePosition(float elapsedMillis) {
		// measure amount of ground covered
		Ptr<Vec3> distVec = velocity_->mult(elapsedMillis);
		float dist = distVec->mag();

		if (dist > 0) {
			// amount of ground covered = arc length of roll --> compute angle
			// thetaRadians = arc length / radius
			float angle = dist / 0.5;
			angle = Math::degrees(angle);

			// rotation axis perpendicular to velocity
			// NOTE: assuming enemy only moves in x-y plane
			Ptr<Vec3> negZ = new Vec3(0, 0, -1);
			Ptr<Vec3> rotAxis = velocity_->cross(negZ);

			//sceneNode_->rotate(angle, rotAxis->x, rotAxis->y, rotAxis->z);
			sceneNode_->translate(distVec->x, distVec->y, distVec->z);
		}
	}

	Ptr<Vertex3> Enemy::position() {
		return sceneNode_->worldPosition();
	}

	bool Enemy::getSmacked(Ptr<Projectile> projectile) {
		enemyData_->healthDec(projectile->damage());
		if (enemyData_->health() <= 0) {
			del();
			return true;
		}
		return false;
	}

	void Light::draw() {
		glPushMatrix();

		glRotatef(angle_, 0, 0, 1);

		if (hide_) {
			glTranslatef(0, 0, -5);
		} else {
			glTranslatef(-3, 0, 0);
		}
/*		glTranslatef(
			-sceneNode_->worldPosition()->x,
			-sceneNode_->worldPosition()->y,
			-sceneNode_->worldPosition()->z);
*/
		float lightPos[4] = { 0, 0, 0, 1 };
		glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
/*		
		glDisable(GL_LIGHTING);
		glColor4f(sphereColor_->r, sphereColor_->g, sphereColor_->b, sphereColor_->a);

		GLUquadric *sphere = gluNewQuadric();
		gluQuadricTexture(sphere, true);
		gluSphere(sphere, 0.25, 20, 20);
		gluDeleteQuadric(sphere);
*/
		glPopMatrix();
	}
}
