#include <SDL/SDL_opengl.h>
#include <iostream>
#include <sstream>

#include "TowerHUD.h"
#include "TowerColor.h"

namespace Tower {

	void TowerHUD::init() {
		Ptr<Panel> panel;
		Ptr<Label> l;
		Ptr<Button> b;

		//rootPanel_ = new TowerRootPanel(rootPanel_->parent(), this);
		rootPanel_ = new Panel(NULL);
		rootPanel_->xIs(winWidth_ - winWidth_ * 0.3);
		rootPanel_->yIs(winHeight_);
		rootPanel_->widthIs(winWidth_ * 0.3);
		rootPanel_->heightIs(winHeight_);
		rootPanel_->backgroundColorIs(new Color4(0.5, 0.5, 0.5, 0.5));

		// gold
		l = new Label(rootPanel_);
		l->xIs(rootPanel_->x() + 30);
		l->yIs(rootPanel_->y() - 50);
		l->widthIs(rootPanel_->width() * 0.7);
		l->heightIs(30);
		l->backgroundColorIs(new Color4(0.7, 0.7, 0.7, 0.5));
		l->textIs("Gold: 0");
		l->textxIs(l->x() + 4);
		l->textyIs(l->y() - l->textHeight() - 4);

		goldLabel_ = l;

		// lives
		l = new Label(rootPanel_);
		l->xIs(rootPanel_->x() + 30);
		l->yIs(rootPanel_->y() - 100);
		l->widthIs(rootPanel_->width() * 0.7);
		l->heightIs(30);
		l->backgroundColorIs(new Color4(0.7, 0.7, 0.7, 0.5));
		l->textIs("Lives: 0");
		l->textxIs(l->x() + 4);
		l->textyIs(l->y() - l->textHeight() - 4);

		livesLabel_ = l;

		// level
		l = new Label(rootPanel_);
		l->xIs(rootPanel_->x() + 30);
		l->yIs(rootPanel_->y() - 150);
		l->widthIs(rootPanel_->width() * 0.7);
		l->heightIs(30);
		l->backgroundColorIs(new Color4(0.7, 0.7, 0.7, 0.5));
		l->textIs("Level: 0");
		l->textxIs(l->x() + 4);
		l->textyIs(l->y() - l->textHeight() - 4);

		levelLabel_ = l;

		// towers
		panel = new Panel(rootPanel_);
		panel->xIs(rootPanel_->x() + 30);
		panel->yIs(rootPanel_->y() - 200);
		panel->widthIs(rootPanel_->width() * 0.7);
		panel->heightIs(150);
		panel->backgroundColorIs(new Color4(0.7, 0.7, 0.7, 0.5));

		l = new Label(panel);
		l->xIs(panel->x());
		l->yIs(panel->y());
		l->textIs("Towers");
		l->textxIs(l->x() + 4);
		l->textyIs(l->y() - l->textHeight() - 4);

		// fire tower
		b = new TowerButton(panel, TowerData::FireTowerType, this);
		b->xIs(panel->x() + 4);
		b->yIs(panel->y() - 30);
		b->widthIs(50);
		b->heightIs(50);
		b->backgroundColorIs(TowerColor::Fire());
		b->textIs("Fire");
		b->textxIs(b->x() + 4);
		b->textyIs(b->y() - b->textHeight() - 4);

		fireButton_ = b;

		// ice tower
		b = new TowerButton(panel, TowerData::IceTowerType, this);
		b->xIs(panel->x() + 59);
		b->yIs(panel->y() - 30);
		b->widthIs(50);
		b->heightIs(50);
		b->backgroundColorIs(TowerColor::Ice());
		b->textIs("Ice");
		b->textxIs(b->x() + 4);
		b->textyIs(b->y() - b->textHeight() - 4);

		iceButton_ = b;

		// earth tower
		b = new TowerButton(panel, TowerData::EarthTowerType, this);
		b->xIs(panel->x() + 114);
		b->yIs(panel->y() - 30);
		b->widthIs(50);
		b->heightIs(50);
		b->backgroundColorIs(TowerColor::Earth());
		b->textIs("Earth");
		b->textxIs(b->x() + 4);
		b->textyIs(b->y() - b->textHeight() - 4);

		earthButton_ = b;

		// context sensitive area
		panel = new Panel(rootPanel_);
		panel->xIs(rootPanel_->x() + 30);
		panel->yIs(rootPanel_->y() - 370);
		panel->widthIs(rootPanel_->width() * 0.7);
		panel->heightIs(270);
		panel->backgroundColorIs(new Color4(0.7, 0.7, 0.7, 0.5));

		contextPanel_ = panel;

		// start button
		b = new StartButton(rootPanel_, this);
		b->xIs(rootPanel_->x() + 30);
		b->yIs(rootPanel_->y() - 660);
		b->widthIs(rootPanel_->width() * 0.7);
		b->heightIs(50);
		b->backgroundColorIs(new Color4(0.8, 0.8, 0.5, 0.5));
		b->fontIs(GLUT_BITMAP_HELVETICA_18);
		b->textIs("START");
		b->textxIs(b->x() + 80);
		b->textyIs(b->y() - b->textHeight() - 14);

		startButton_ = b;
	}

	void TowerHUD::updateText() {
		stringstream ss;
		ss << engine_->towerEngineData()->gold();
		goldLabel_->textIs("Gold: " + ss.str());

		ss.str("");
		ss << engine_->towerEngineData()->lives();
		livesLabel_->textIs("Lives: " + ss.str());

		ss.str("");
		ss << engine_->towerEngineData()->level();
		levelLabel_->textIs("Level: " + ss.str());
	}

	void TowerHUD::draw() {
		setupOrthoView(winWidth_, winHeight_);

		updateText();
		rootPanel_->draw();

		setupPerspectiveView();
	}

	void TowerHUD::contextInfoReset() {
		contextPanel_->clear();
	}
	void TowerHUD::contextTowerInfo(TowerData::TowerType towerType) {
		Ptr<TowerData> towerData = engine_->towerData(towerType);

		Ptr<Panel> p = new Panel(contextPanel_);
		p->xIs(contextPanel_->x() + 4);
		p->yIs(contextPanel_->y() - 15);
		p->widthIs(contextPanel_->width() * 0.94);
		p->heightIs(contextPanel_->height() * 0.87);
		p->backgroundColorIs(new Color4(0.8, 0.8, 0.8, 0.5));

		// tower type
		stringstream ss;
		if (towerType == TowerData::FireTowerType) {
			ss << "Fire Tower";
		} else if (towerType == TowerData::IceTowerType) {
			ss << "Ice Tower";
		} else if (towerType == TowerData::EarthTowerType) {
			ss << "Earth Tower";
		}

		Ptr<Label> tl = new Label(p);
		tl->xIs(p->x() + 4);
		tl->yIs(p->y() - 15);
		tl->widthIs(p->width() * 0.8);
		tl->heightIs(30);
		tl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		tl->textxIs(tl->x() + 4);
		tl->textyIs(tl->y() - tl->textHeight() - 4);
		tl->fontIs(GLUT_BITMAP_HELVETICA_18);
		tl->textIs(ss.str());

		// gold
		ss.str("");
		ss << "Cost: " << towerData->cost();

		Ptr<Label> gl = new Label(p);
		gl->xIs(tl->x());
		gl->yIs(tl->y() - tl->height() - 10);
		gl->widthIs(p->width() * 0.8);
		gl->heightIs(30);
		gl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		gl->textxIs(gl->x() + 4);
		gl->textyIs(gl->y() - gl->textHeight() - 4);
		gl->fontIs(GLUT_BITMAP_HELVETICA_18);
		gl->textIs(ss.str());

		// damage
		ss.str("");
		ss << "Damage: " << towerData->damage();

		Ptr<Label> dl = new Label(p);
		dl->xIs(gl->x());
		dl->yIs(gl->y() - gl->height() - 10);
		dl->widthIs(p->width() * 0.8);
		dl->heightIs(30);
		dl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		dl->textxIs(dl->x() + 4);
		dl->textyIs(dl->y() - dl->textHeight() - 4);
		dl->fontIs(GLUT_BITMAP_HELVETICA_18);
		dl->textIs(ss.str());

		// range
		ss.str("");
		ss << "Range: " << towerData->range();

		Ptr<Label> rl = new Label(p);
		rl->xIs(dl->x());
		rl->yIs(dl->y() - dl->height() - 10);
		rl->widthIs(p->width() * 0.8);
		rl->heightIs(30);
		rl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		rl->textxIs(rl->x() + 4);
		rl->textyIs(rl->y() - rl->textHeight() - 4);
		rl->fontIs(GLUT_BITMAP_HELVETICA_18);
		rl->textIs(ss.str());
	}

	void TowerHUD::contextTowerUpgradeInfo(Ptr<Tower> tower) {
		Ptr<TowerData> towerData = tower->towerData();
		Ptr<TowerUpgradeData> upgradeData = towerData->upgradeNextData();

		// stats/upgrade
		stringstream ss;
		ss << "Stats";

		Ptr<Label> ul = new Label(contextPanel_);
		ul->xIs(contextPanel_->x() + 4);
		ul->yIs(contextPanel_->y() - 15);
		ul->widthIs(contextPanel_->width() * 0.9);
		ul->heightIs(30);
		ul->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		ul->textxIs(ul->x() + 4);
		ul->textyIs(ul->y() - ul->textHeight() - 4);
		ul->fontIs(GLUT_BITMAP_HELVETICA_18);
		ul->textIs(ss.str());

		// tower type
		ss.str("");
		ss << " L" << (towerData->upgradeLevel() + 1);
		if (towerData->towerType() == TowerData::FireTowerType) {
			ss << "Fire";
		} else if (towerData->towerType() == TowerData::IceTowerType) {
			ss << "Ice";
		} else if (towerData->towerType() == TowerData::EarthTowerType) {
			ss << "Earth";
		}

		Ptr<Label> tl = new Label(contextPanel_);
		tl->xIs(ul->x());
		tl->yIs(ul->y() - ul->height() - 10);
		tl->widthIs(contextPanel_->width() * 0.9);
		tl->heightIs(30);
		tl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		tl->textxIs(tl->x() + 4);
		tl->textyIs(tl->y() - tl->textHeight() - 4);
		tl->fontIs(GLUT_BITMAP_HELVETICA_18);
		tl->textIs(ss.str());

		// damage
		ss.str("");
		ss << "Damage: " << towerData->damage();
		if (upgradeData != NULL) {
			ss << " (+" << upgradeData->damage() << ")";
		}

		Ptr<Label> dl = new Label(contextPanel_);
		dl->xIs(tl->x());
		dl->yIs(tl->y() - tl->height() - 10);
		dl->widthIs(contextPanel_->width() * 0.9);
		dl->heightIs(30);
		dl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		dl->textxIs(dl->x() + 4);
		dl->textyIs(dl->y() - dl->textHeight() - 4);
		dl->fontIs(GLUT_BITMAP_HELVETICA_18);
		dl->textIs(ss.str());

		// range
		ss.str("");
		ss << "Range: " << towerData->range();
		if (upgradeData != NULL) {
			ss << " (+" << upgradeData->range() << ")";
		}

		Ptr<Label> rl = new Label(contextPanel_);
		rl->xIs(dl->x());
		rl->yIs(dl->y() - dl->height() - 10);
		rl->widthIs(contextPanel_->width() * 0.9);
		rl->heightIs(30);
		rl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
		rl->textxIs(rl->x() + 4);
		rl->textyIs(rl->y() - rl->textHeight() - 4);
		rl->fontIs(GLUT_BITMAP_HELVETICA_18);
		rl->textIs(ss.str());

		if (upgradeData != NULL) {
			// cost
			ss.str("");
			ss << "Cost: " << upgradeData->cost();

			Ptr<Label> cl = new Label(contextPanel_);
			cl->xIs(rl->x());
			cl->yIs(rl->y() - rl->height() - 10);
			cl->widthIs(contextPanel_->width() * 0.9);
			cl->heightIs(30);
			cl->backgroundColorIs(new Color4(0.5, 0.8, 0.8, 0.5));
			cl->textxIs(cl->x() + 4);
			cl->textyIs(cl->y() - cl->textHeight() - 4);
			cl->fontIs(GLUT_BITMAP_HELVETICA_18);
			cl->textIs(ss.str());

			// upgrade button
			ss.str("");
			ss << "Upgrade";

			Ptr<UpgradeButton> b = new UpgradeButton(contextPanel_, this);
			b->xIs(cl->x());
			b->yIs(cl->y() - cl->height() - 10);
			b->widthIs(contextPanel_->width() * 0.9);
			b->heightIs(30);
			b->backgroundColorIs(new Color4(0.8, 0.8, 0.5, 0.5));
			b->textxIs(b->x() + 30);
			b->textyIs(b->y() - b->textHeight() - 4);
			b->fontIs(GLUT_BITMAP_HELVETICA_18);
			b->textIs(ss.str());

			b->enabledIs(upgradeEnabled_);
			upgradeButton_ = b;
		}
	}

	void TowerHUD::upgradeSelectedTower() {
		Ptr<TowerEngineData> engineData = engine_->towerEngineData();
		Ptr<Tower> tower = engineData->selectedTower();
		Ptr<TowerUpgradeData> upgradeData = tower->towerData()->upgradeNextData();

		if (upgradeData->cost() <= engineData->gold()) {
			tower->towerData()->upgrade();
			engineData->goldDec(upgradeData->cost());
			engineData->selectedTowerIs(NULL);
			contextInfoReset();
		}
	}

	void TowerHUD::onReset() {
		startButton_->visibleIs(true);
	}

	void TowerHUD::onRoundOver() {
		onReset();
	}

	void TowerHUD::updateTowerAndUpgradeButtons(bool enabled) {
		ptr_cast<TowerButton>(fireButton_)->enabledIs(enabled);
		ptr_cast<TowerButton>(iceButton_)->enabledIs(enabled);
		ptr_cast<TowerButton>(earthButton_)->enabledIs(enabled);
		upgradeEnabled_ = enabled;
		if (upgradeButton_) {
			ptr_cast<UpgradeButton>(upgradeButton_)->enabledIs(enabled);
		}
	}
	
	//
	// TowerButton
	//

	void TowerButton::onMouseEnter(int x, int y) {
		if (enabled_) {
			Ptr<Color4> c;
			if (towerType_ == TowerData::FireTowerType) {
				c = TowerColor::Fire();
			} else if (towerType_ == TowerData::IceTowerType) {
				c = TowerColor::Ice();
			} else if (towerType_ == TowerData::EarthTowerType) {
				c = TowerColor::Earth();
			}
			c->r += 0.1;
			c->g += 0.1;
			c->b += 0.1;
			backgroundColorIs(c);
		}
	}

	void TowerButton::onMouseExit(int x, int y) {
		if (enabled_) {
			if (towerType_ == TowerData::FireTowerType) {
				backgroundColorIs(TowerColor::Fire());
			} else if (towerType_ == TowerData::IceTowerType) {
				backgroundColorIs(TowerColor::Ice());
			} else if (towerType_ == TowerData::EarthTowerType) {
				backgroundColorIs(TowerColor::Earth());
			}
		}
	}

	bool TowerButton::ML::onMouseUp(int sdlButton, int x, int y) {
		return towerButton_->contains(x, y);
	}
	bool TowerButton::ML::onMouseDown(int sdlButton, int x, int y) {
		if (sdlButton == SDL_BUTTON_LEFT &&
			towerButton_->contains(x, y) &&
			towerButton_->enabled_) {
			towerButton_->hud_->contextTowerInfo(towerButton_->towerType_);
			towerButton_->hud_->newTowerTypeIs(towerButton_->towerType_);
		}
		return towerButton_->contains(x, y);
	}
	bool TowerButton::ML::onMouseMotion(int x, int y) {
		if (towerButton_->contains(x, y)) {
			if (!towerButton_->hover_) {
				towerButton_->hover_ = true;
				towerButton_->onMouseEnter(x, y);
			}
		} else if (towerButton_->hover_) {
			towerButton_->hover_ = false;
			towerButton_->onMouseExit(x, y);
		}
		return towerButton_->contains(x, y);
	}

	void TowerButton::enabledIs(bool v) {
		enabled_ = v;
		if (enabled_) {
			if (towerType_ == TowerData::FireTowerType) {
				backgroundColorIs(TowerColor::Fire());
			} else if (towerType_ == TowerData::IceTowerType) {
				backgroundColorIs(TowerColor::Ice());
			} else if (towerType_ == TowerData::EarthTowerType) {
				backgroundColorIs(TowerColor::Earth());
			}
		} else {
			backgroundColorIs(TowerColor::Gray());
		}
	}

	//
	// UpgradeButton
	//

	void UpgradeButton::onMouseEnter(int x, int y) {
		if (enabled_) {
			backgroundColorIs(new Color4(0.9, 0.9, 0.6, 1));
		}
	}

	void UpgradeButton::onMouseExit(int x, int y) {
		if (enabled_) {
			backgroundColorIs(new Color4(0.8, 0.8, 0.5, 0.5));
		}
	}

	bool UpgradeButton::ML::onMouseUp(int sdlButton, int x, int y) {
		return upgradeButton_->contains(x, y);
	}

	bool UpgradeButton::ML::onMouseDown(int sdlButton, int x, int y) {
		if (sdlButton == SDL_BUTTON_LEFT &&
			upgradeButton_->contains(x, y) &&
			upgradeButton_->enabled_) {
			upgradeButton_->hud_->upgradeSelectedTower();
		}
		return upgradeButton_->contains(x, y);
	}

	bool UpgradeButton::ML::onMouseMotion(int x, int y) {
		if (upgradeButton_->contains(x, y)) {
			if (!upgradeButton_->hover_) {
				upgradeButton_->hover_ = true;
				upgradeButton_->onMouseEnter(x, y);
			}
		} else if (upgradeButton_->hover_) {
			upgradeButton_->hover_ = false;
			upgradeButton_->onMouseExit(x, y);
		}
		return upgradeButton_->contains(x, y);
	}

	void UpgradeButton::enabledIs(bool v) {
		enabled_ = v;
		if (enabled_) {
			backgroundColorIs(new Color4(0.8, 0.8, 0.5, 1));
		} else {
			backgroundColorIs(new Color4(0.5, 0.5, 0.5, 1));
		}
	}

	//
	// StartButton
	//

	void StartButton::onMouseEnter(int x, int y) {
		Ptr<Color4> c = new Color4(0.8, 0.8, 0.5, 0.5);
		c->r += 0.1;
		c->g += 0.1;
		c->b += 0.1;
		backgroundColorIs(c);
	}

	void StartButton::onMouseExit(int x, int y) {
		backgroundColorIs(new Color4(0.8, 0.8, 0.5, 0.5));
	}

	bool StartButton::ML::onMouseUp(int sdlButton, int x, int y) {
		return startButton_->contains(x, y);
	}

	bool StartButton::ML::onMouseDown(int sdlButton, int x, int y) {
		if (sdlButton == SDL_BUTTON_LEFT && startButton_->contains(x, y)) {
			startButton_->visibleIs(false);
			startButton_->hud_->engine()->startRound();
		}
		return startButton_->contains(x, y);
	}

	bool StartButton::ML::onMouseMotion(int x, int y) {
		if (startButton_->contains(x, y)) {
			if (!startButton_->hover_) {
				startButton_->hover_ = true;
				startButton_->onMouseEnter(x, y);
			}
		} else if (startButton_->hover_) {
			startButton_->hover_ = false;
			startButton_->onMouseExit(x, y);
		}
		return startButton_->contains(x, y);
	}

}
