#ifndef TOWERHUD_H
#define TOWERHUD_H

#include "Ptr.h"
#include "PtrInterface.h"
#include "TowerEngine.h"
#include "HUD.h"

namespace Tower {

	using namespace Fwk;
	using namespace std;
	using namespace GameEngine;

	class TowerHUD : public HUD {
	public:
		TowerHUD(Ptr<TowerEngine> engine, int winWidth, int winHeight) :
		  HUD(winWidth, winHeight),
		  engine_(engine),
		  newTowerType_(TowerData::NoTowerType),
		  upgradeEnabled_(true) {
			init();
		}

		virtual void build() { };
		virtual void draw();

		void contextInfoReset();
		void contextTowerInfo(TowerData::TowerType towerType);
		void contextTowerUpgradeInfo(Ptr<Tower> tower);
		TowerData::TowerType newTowerType() { return newTowerType_; }
		void newTowerTypeIs(TowerData::TowerType type) { newTowerType_ = type; }
		Ptr<TowerEngine> engine() { return engine_; }
		void upgradeSelectedTower();
		void onReset();
		void onRoundOver();
		void updateTowerAndUpgradeButtons(bool enabled);

	protected:
		Ptr<TowerEngine> engine_;
		TowerData::TowerType newTowerType_;

		Ptr<Label> goldLabel_;
		Ptr<Label> livesLabel_;
		Ptr<Label> levelLabel_;
		Ptr<Button> fireButton_;
		Ptr<Button> iceButton_;
		Ptr<Button> earthButton_;
		bool upgradeEnabled_;
		Ptr<Button> upgradeButton_;
		Ptr<Panel> contextPanel_;
		Ptr<Button> startButton_;

		void init();
		void updateText();
	};

	class TowerButton : public Button {
	public:

		// inner class
		class ML : public MouseListener {
		public:
			ML(Ptr<TowerButton> button) : towerButton_(button) { }
			bool onMouseUp(int sdlButton, int x, int y);
			bool onMouseDown(int sdlButton, int x, int y);
			bool onMouseMotion(int x, int y);
		protected:
			Ptr<TowerButton> towerButton_;
		};

		TowerButton(Ptr<Container> parent, TowerData::TowerType towerType, Ptr<TowerHUD> hud) :
		  Button(parent),
		  towerType_(towerType),
		  hud_(hud),
		  mouseListener_(new ML(this)),
		  enabled_(true) {
			hud_->mouseListenerNew(mouseListener_);
		}

		virtual void onDel() { hud_->mouseListenerDel(mouseListener_); }

		void enabledIs(bool v);

	protected:
		TowerData::TowerType towerType_;
		Ptr<TowerHUD> hud_;
		Ptr<MouseListener> mouseListener_;
		bool enabled_;

		virtual void onMouseEnter(int x, int y);
		virtual void onMouseExit(int x, int y);
	};

	class UpgradeButton : public Button {
	public:

		// inner class
		class ML : public MouseListener {
		public:
			ML(Ptr<UpgradeButton> button) : upgradeButton_(button) { }
			bool onMouseUp(int sdlButton, int x, int y);
			bool onMouseDown(int sdlButton, int x, int y);
			bool onMouseMotion(int x, int y);
		protected:
			Ptr<UpgradeButton> upgradeButton_;
		};

		UpgradeButton(Ptr<Container> parent, Ptr<TowerHUD> hud) :
		  Button(parent),
		  hud_(hud),
		  mouseListener_(new ML(this)),
		  enabled_(true) {
			hud_->mouseListenerNew(mouseListener_);
		}

		virtual void onDel() { hud_->mouseListenerDel(mouseListener_); }

		void enabledIs(bool v);

	protected:
		Ptr<TowerHUD> hud_;
		Ptr<MouseListener> mouseListener_;
		bool enabled_;

		virtual void onMouseEnter(int x, int y);
		virtual void onMouseExit(int x, int y);
	};

	class StartButton : public Button {
	public:

		// inner class
		class ML : public MouseListener {
		public:
			ML(Ptr<StartButton> button) : startButton_(button) { }
			bool onMouseUp(int sdlButton, int x, int y);
			bool onMouseDown(int sdlButton, int x, int y);
			bool onMouseMotion(int x, int y);
		protected:
			Ptr<StartButton> startButton_;
		};

		StartButton(Ptr<Container> parent, Ptr<TowerHUD> hud) :
		  Button(parent),
		  hud_(hud),
		  mouseListener_(new ML(this)) {
			hud_->mouseListenerNew(mouseListener_);
		}

		virtual void onDel() { hud_->mouseListenerDel(mouseListener_); }

	protected:
		Ptr<TowerHUD> hud_;
		Ptr<MouseListener> mouseListener_;

		virtual void onMouseEnter(int x, int y);
		virtual void onMouseExit(int x, int y);
	};

}

#endif
