#include "TowerParticle.h"
#include "TowerColor.h"
#include "TextureManager.h"

namespace Tower {

	void DustSystem::init() {
		int numParticles = 5 + projectile_->tower()->towerData()->upgradeLevel() * 3;
		float lifeInSeconds = 1 + projectile_->tower()->towerData()->upgradeLevel() * 0.5;
		float size = 0.8 + projectile_->tower()->towerData()->upgradeLevel() * 0.3;
		float speed = 0.001;
		Ptr<RandomNumberGenerator> randNumGen = RandomNumberGenerator::Instance();
/*
		if (totalNumParticles_ == 0) {
			zlevel_ = 0;
		}
*/
		for (int i = 0; i < numParticles; i++) {
			Ptr<Particle> p = new Particle();
			particles_.push_back(p);

			//totalNumParticles_++;
			//zlevel_ += Z_INC;

			// scene node
			Ptr<DrawableSceneNode> sn = new DrawableSceneNode(p);
			p->sceneNodeIs(sn);
			sceneNode_->childNew(sn);

			// life
			p->lifeIs(lifeInSeconds * 1000);

			// position
			float xdir = randNumGen->neg1to1() * size / 2;
			float ydir = randNumGen->neg1to1() * size / 2;
			Ptr<Vertex3> o = new Vertex3(origin_->x + xdir, origin_->y + ydir, origin_->z);
			//o->z += zlevel_;
			p->positionIs(o);

			// size
			p->sizeIs(size);

			// color
			p->colorIs(TowerColor::Dust());

			// texture
			p->textureIs(TextureManager::Instance()->texture(TextureManager::GLOW_IMAGE_FILE));
			//p->textureIs(TextureManager::Instance()->texture(TextureManager::HARDGLOW_IMAGE_FILE));

			// velocity
			Ptr<Vec3> vel = new Vec3(projectile_->velocity());
			vel->normalize();
			vel->x = vel->x * 0.5 + randNumGen->neg1to1() * 0.5;
			vel->y = vel->y * 0.5 + randNumGen->neg1to1() * 0.5;
			vel->z = 0;
			vel->normalize();
			speed *= 1.0 / p->size();
			vel = vel->mult(speed);
			p->velocityIs(vel);
		}
	}

	void DustSystem::update(float elapsedMillis) {
		Ptr<Particle> p;

		// update particle stats
		vector<Ptr<Particle> >::iterator pIt = particles_.begin();
		while (pIt != particles_.end()) {
			p = *pIt;

			// life
			p->lifeDec(elapsedMillis);

			// position
			Ptr<Vec3> off = p->velocity()->mult(elapsedMillis);
			p->sceneNode()->translate(off->x, off->y, off->z);

			// size

			// color
			float alpha = p->lifePercent();
			Ptr<Color4> c = p->color();
			c->a = alpha;
			p->colorIs(c);

			pIt++;
		}
		removeDeadParticles();
	}

	void FireSystem::init() {
		int numParticles = 5 + projectile_->tower()->towerData()->upgradeLevel() * 3;
		float lifeInSeconds = 1 + projectile_->tower()->towerData()->upgradeLevel() * 0.6;
		float size = 0.8 + projectile_->tower()->towerData()->upgradeLevel() * 0.5;
		float speed = 0.001;
		Ptr<RandomNumberGenerator> randNumGen = RandomNumberGenerator::Instance();
/*
		if (totalNumParticles_ == 0) {
			zlevel_ = 0;
		}
*/
		for (int i = 0; i < numParticles; i++) {
			Ptr<Particle> p = new Particle();
			particles_.push_back(p);
/*
			totalNumParticles_++;
			zlevel_ += Z_INC;
*/
			// scene node
			Ptr<DrawableSceneNode> sn = new DrawableSceneNode(p);
			p->sceneNodeIs(sn);
			sceneNode_->childNew(sn);

			// life
			p->lifeIs(lifeInSeconds * 1000);

			// position
			float xdir = randNumGen->neg1to1() * size / 2;
			float ydir = randNumGen->neg1to1() * size / 2;
			Ptr<Vertex3> o = new Vertex3(origin_->x + xdir, origin_->y + ydir, origin_->z);
			//o->z += zlevel_;
			p->positionIs(o);

			// size
			p->sizeIs(size);

			// color
			p->colorIs(TowerColor::Fire());

			// texture
			p->textureIs(TextureManager::Instance()->texture(TextureManager::GLOW_IMAGE_FILE));
			//p->textureIs(TextureManager::Instance()->texture(TextureManager::HARDGLOW_IMAGE_FILE));

			// velocity
			Ptr<Vec3> vel = new Vec3(projectile_->velocity());
			vel->normalize();
			vel->x = vel->x * 0.5 + randNumGen->neg1to1() * 0.5;
			vel->y = vel->y * 0.5 + randNumGen->neg1to1() * 0.5;
			vel->z = 0;
			vel->normalize();
			speed *= 1.0 / p->size();
			vel = vel->mult(speed);
			p->velocityIs(vel);
		}
	}

	float mymax(float a, float b) {
		return a > b ? a : b;
	}

	void FireSystem::update(float elapsedMillis) {
		Ptr<Particle> p;

		// update particle stats
		vector<Ptr<Particle> >::iterator pIt = particles_.begin();
		while (pIt != particles_.end()) {
			p = *pIt;

			// life
			p->lifeDec(elapsedMillis);

			// position
			Ptr<Vec3> off = p->velocity()->mult(elapsedMillis);
			p->sceneNode()->translate(off->x, off->y, off->z);

			// size

			// color
			float alpha = p->lifePercent();
			Ptr<Color4> c = p->color();
			c->r = mymax(0.6, c->r * p->lifePercent());
			c->g = mymax(0.2, c->g * p->lifePercent());
			c->b = mymax(0.2, c->b * p->lifePercent());
			c->a = alpha;
			p->colorIs(c);

			pIt++;
		}
		removeDeadParticles();
	}

	void IceSystem::init() {
		int numParticles = 15;
		float lifeInSeconds = 2;
		float size = 0.5;
		float speed = 0.0015;
		Ptr<RandomNumberGenerator> randNumGen = RandomNumberGenerator::Instance();

		for (int i = 0; i < numParticles; i++) {
			Ptr<Particle> p = new Particle();
			particles_.push_back(p);

			// scene node
			Ptr<DrawableSceneNode> sn = new DrawableSceneNode(p);
			p->sceneNodeIs(sn);
			sceneNode_->childNew(sn);

			// life
			p->lifeIs(lifeInSeconds * 1000);

			// position
			p->positionIs(origin_);

			// size
			p->sizeIs(size);

			// color
			p->colorIs(TowerColor::Ice());

			// velocity
			float xdir = 1 - randNumGen->gen() * 2;
			float ydir = 1 - randNumGen->gen() * 2;
			Ptr<Vec3> vel = new Vec3(xdir, ydir, 0);
			vel->normalize();
			vel = vel->mult(speed);
			p->velocityIs(vel);
		}
	}

	void IceSystem::update(float elapsedMillis) {
		Ptr<Particle> p;

		// update particle stats
		vector<Ptr<Particle> >::iterator pIt = particles_.begin();
		while (pIt != particles_.end()) {
			p = *pIt;

			// life
			p->lifeDec(elapsedMillis);

			// position
			Ptr<Vec3> off = p->velocity()->mult(elapsedMillis);
			p->sceneNode()->translate(off->x, off->y, off->z);

			// size

			// color
			float alpha = p->lifePercent();
			Ptr<Color4> c = p->color();
			c->a = alpha;
			p->colorIs(c);

			pIt++;
		}
		removeDeadParticles();
	}

	void IceTrailSystem::init() {
		timer_->start();
		addParticles();
	}

	void IceTrailSystem::addParticles() {
		int numParticles = 1;
		float lifeInSeconds = 1;
		float degreesRange = 20 + projectile_->tower()->towerData()->upgradeLevel() * 10;
		float speedScale = 0.7;
		float size = 0.2 + projectile_->tower()->towerData()->upgradeLevel() * 0.3;
		Ptr<RandomNumberGenerator> randNumGen = RandomNumberGenerator::Instance();
/*
		if (totalNumParticles_ == 0) {
			zlevel_ = 0;
		}
*/
		for (int i = 0; i < numParticles; i++) {
			Ptr<Particle> p = new Particle();
			particles_.push_back(p);

			p->billboardIs(billboardTrue_);
/*
			totalNumParticles_++;
			zlevel_ += Z_INC;
*/
			// scene node
			Ptr<DrawableSceneNode> sn = new DrawableSceneNode(p);
			p->sceneNodeIs(sn);
			sceneNode_->childNew(sn);

			// life
			p->lifeIs(lifeInSeconds * 1000);

			// position
			Ptr<Vertex3> pos = projectile_->position();
			Ptr<Vertex3> o = new Vertex3(pos->x, pos->y, pos->z);
			//o->z += zlevel_;
			p->positionIs(o);

			// size
			p->sizeIs(size);

			// color
			p->colorIs(TowerColor::Ice());

			// texture
			//p->textureIs(TextureManager::Instance()->texture(TextureManager::GLOW_IMAGE_FILE));
			p->textureIs(TextureManager::Instance()->texture(TextureManager::HARDGLOW_IMAGE_FILE));

			// velocity
			Ptr<Vec3> vel = new Vec3(projectile_->velocity());
			float speed = vel->mag() * (1.0f / p->size()) * speedScale;

			// add some jitter:
			float theta = randNumGen->neg1to1() * Math::radians(degreesRange);
			float vangle = 0;
			if (vel->x != 0) {
				vangle = atanf(vel->y / vel->x);
				if (vel->x < 0 && vel->y >= 0) vangle += Math::radians(180);
				else if (vel->x < 0 && vel->y <= 0) vangle += Math::radians(180);
			} else {
				if (vel->y > 0) vangle += Math::radians(90);
				else vangle += Math::radians(-90);
			}
			theta += vangle;
			vel->x = cosf(theta);
			vel->y = sinf(theta);

			vel->normalize();
			vel = vel->mult(speed);
			p->velocityIs(vel);
		}
	}

	void IceTrailSystem::update(float elapsedMillis) {
		float genRateInSecs = 0.02 - projectile_->tower()->towerData()->upgradeLevel() * 0.005;
		float genRate = genRateInSecs * 1000;

		// update particle stats
		vector<Ptr<Particle> >::iterator pIt = particles_.begin();
		while (pIt != particles_.end()) {
			Ptr<Particle> p = *pIt;

			// life
			p->lifeDec(elapsedMillis);

			// position
			Ptr<Vec3> off = p->velocity()->mult(elapsedMillis);
			p->sceneNode()->translate(off->x, off->y, off->z);

			// size

			// color
			float alpha = p->lifePercent();
			Ptr<Color4> c = p->color();
			c->a = alpha;
			p->colorIs(c);

			// speed
			Ptr<Vec3> v = p->velocity();
			float speed = v->mag() * p->lifePercent();
			v->normalize();
			v = v->mult(speed);
			p->velocityIs(v);

			pIt++;
		}

		removeDeadParticles();

		if (!projectile_->dead()) {
			// add particles on a timer
			unsigned int ticks = timer_->elapsedMillis();
			if (ticks > genRate) {
				addParticles();
				timer_->setTicks(ticks - genRate);
			}
		}
		timer_->update();
	}
}
