#ifndef TOWERPARTICLE_H
#define TOWERPARTICLE_H

#include "Ptr.h"
#include "PtrInterface.h"
#include "Timer.h"
#include "Particle.h"
#include "Tower.h"

namespace Tower {

	using namespace Fwk;
	using namespace std;

	//
	// DustSystem
	//

	class DustSystem : public ParticleSystem {
	public:
		DustSystem(Ptr<Engine> engine, Ptr<Projectile> projectile, Ptr<SceneNode> parentSN) :
		  ParticleSystem(engine, projectile->position(), parentSN),
		  projectile_(projectile) { }

		virtual void init();
		virtual void update(float elapsedMillis);
	protected:
		Ptr<Projectile> projectile_;
	};

	//
	// FireSystem
	//

	class FireSystem : public ParticleSystem {
	public:
		FireSystem(Ptr<Engine> engine, Ptr<Projectile> projectile, Ptr<SceneNode> parentSN) :
		  ParticleSystem(engine, projectile->position(), parentSN),
		  projectile_(projectile) { }

		virtual void init();
		virtual void update(float elapsedMillis);
	protected:
		Ptr<Projectile> projectile_;
	};

	//
	// IceSystem
	//

	class IceSystem : public ParticleSystem {
	public:
		IceSystem(Ptr<Engine> engine, Ptr<Vertex3> origin, Ptr<SceneNode> parentSN) :
		  ParticleSystem(engine, origin, parentSN) { }

		virtual void init();
		virtual void update(float elapsedMillis);
	};

	//
	// IceTrailSystem
	//

	class IceTrailSystem : public ParticleSystem {
	public:
		IceTrailSystem(Ptr<Engine> engine, Ptr<Projectile> projectile, Ptr<SceneNode> parentSN) :
		  ParticleSystem(engine, projectile->position(), parentSN),
		  projectile_(projectile),
		  timer_(new Timer) { }

		virtual void init();
		virtual void update(float elapsedMillis);

	protected:
		Ptr<Projectile> projectile_;
		Ptr<Timer> timer_;

		void addParticles();
	};

}

#endif